#!/usr/bin/perl

# Provides functions of general utility for the CSM web interface.

BEGIN
  {
	if (!defined($::CSMDIR)) { $::CSMDIR = '.'; }
	$GuiUtils::csmwebroot = "$ENV{PWD}";
	if ($::CSMDIR ne '.') { $GuiUtils::csmwebroot += "/$::CSMDIR"; }
	# this enables us to redirect where it looks for other CSM files during development
	$GuiUtils::csmroot = $ENV{'CSM_ROOT'} ? $ENV{'CSM_ROOT'} : '/opt/csm';

	# The latest HTML versions of the CSM and RSCT books
	$GuiUtils::docRoot = 'http://publib.boulder.ibm.com/clresctr/docs/csm';
	$GuiUtils::installGuideRoot = "$GuiUtils::docRoot/linux/200411/am7il104";
	$GuiUtils::installGuide = "$GuiUtils::installGuideRoot/am7il10402.html";
	$GuiUtils::adminGuideRoot = "$GuiUtils::docRoot/linux/200408/am7al103";
	$GuiUtils::adminGuide = "$GuiUtils::adminGuideRoot/am7al10302.html";
	$GuiUtils::techRefRoot = "$GuiUtils::docRoot/linux/200411/am7cl103";
	$GuiUtils::techRef = "$GuiUtils::techRefRoot/am7cl10302.html";
	$GuiUtils::techRefManPage = "$GuiUtils::techRefRoot/am7cl10304.html";
	# The hw ctrl guide is no longer a separate book.  The planning portions are in the
	# install guide and the operational portions are in the admin guide.  These vars
	# should be removed when no code is referencing them any longer.
	$GuiUtils::hwCtrlGuideRoot = "$GuiUtils::docRoot/linux/200309/am7hl100";
	$GuiUtils::hwCtrlGuide = "$GuiUtils::hwCtrlGuideRoot/am7hl10002.html";
	
	$GuiUtils::rsctDocRoot = 'http://publib.boulder.ibm.com/clresctr/docs/rsct';
	$GuiUtils::rsctAdminGuideRoot = "$GuiUtils::rsctDocRoot/aix_linux/200411/bl5adm06";
	$GuiUtils::rsctAdminGuide = "$GuiUtils::adminGuideRoot/bl5adm0602.html";
	$GuiUtils::rsctTechRefRoot = "$GuiUtils::rsctDocRoot/linux/200411/bl5trl06";
	$GuiUtils::rsctTechRef = "$GuiUtils::techRefRoot/bl5trl0602.html";
  }

use strict;
do "$::CSMDIR/../web-lib.pl";
use lib "$GuiUtils::csmroot/pm";
use CSMDefs;
require NodeUtils;
use Time::Local;
#use File::Basename;

package GuiUtils;


# Reads probemgr output from the filehandle given and displays each line color-coded
# according to the type of output.  This only makes sense if probemgr was run with an
# output level (-l) of 0 or 1 and was run with open to get the filehandle of the output
# (which can be done by passing a mode of 3 into GuiUtils::runcmd).
# If dsh is true, the parsing will account for the hostname that dsh prepends to each line.
sub processProbeOutput
  {
	my ($class, $filehandle, $dsh) = @_;
	print '<P>';
	while(<$filehandle>)
	  {
		chomp($_);
		my $type;
		if ($dsh) { ($type) = $_=~/^.+?:.+?:([^:]+):/; }
		else { ($type) = $_=~/^.+?:([^:]+):/; }
		if ($type eq 'trace') { print "<FONT color='blue'>$_</FONT><BR>\n"; }  #GuiUtils->msg('I', $_); }
		elsif ($type eq 'attention') { print "<FONT color='#999900'><B>$_</B></FONT><BR>\n"; }  #{ GuiUtils->msg('W', $_); }
		elsif ($type eq 'error') { print "<FONT color='red'><B>$_</B></FONT><BR>\n"; }  #GuiUtils->msg('E', $_); }
		else { print "$_<BR>\n"; }
	  }
	print "</P>\n";
  }


# Returns an image name appropriate for the hw type given.  Note that the returned image
# name does not include the directory (usually called images), because the relative directory
# name is different depending on where this is called from.
sub getHWTypeImage
  {
	my ($class, $hwtype) = @_;
	if (grep(/$hwtype/i, qw(x330 x335 8676 e325 8835))) { return '330.gif'; }
	elsif (grep(/$hwtype/i, qw(x340 8656 x342 8669 x345 8670 x360 8686 x365 x445 x450 x455))) { return '342.gif'; }
	elsif (grep(/$hwtype/i, ('HS20 blade', '8832', '8678', 'JS20 blade', '8842'))) { return 'blade.gif'; }
	else { return '330.gif'; }
  }


# Outputs the html appropriate to show the given page heading and node list in a scalable way.
# This should be used for the heading for all pages that operate on nodes.
sub printNodeHeading
  {
	my ($class, $headtxt, $nodestr) = @_;
	$nodestr =~ s/,/, /g;
	#my @nodes = split(/,/, $nodestr);
	if (length($nodestr) > 80)            # put the nodes in a text area
	  {
		print qq(<FORM><B><FONT size="+2">$headtxt</FONT></B><TEXTAREA rows="1" cols="50" readonly name="nodeList" style="vertical-align : middle;">$nodestr</TEXTAREA></FORM>\n);
		#print qq(<FORM><B><FONT size="+2">$headtxt</FONT></B><SELECT name="nodeMenu">\n);
		#foreach my $n (@nodes) { print qq(<OPTION>$n</OPTION>\n); }
		#print qq(</SELECT></FORM>\n);
	  }
	else            # just display the nodes as a string
	  {
		print qq(<P><B><FONT size="+2">$headtxt</FONT><FONT size="+1">$nodestr</FONT></B></P>\n);
	  }
	
	#<FORM><B><FONT size="+2">Run Command on Node(s): </FONT></B><SELECT name="nodeMenu">
	#  <OPTION value="1">node1</OPTION>
	#  <OPTION value="1" selected>node2</OPTION>
	#</SELECT> <B><FONT size="+1">$::in{nodes}</FONT></B></FORM>
  }

# Returns a unique, random filename in /tmp for temporary use.
# If extension is passed in, it is used as the extension for the file name.
sub tmpname
  {
	# could have used the webmin &main::tempname() instead
	my ($class, $extension) = @_;
	if (length($extension)) { $extension = ".$extension"; }
    my $progname = $0;
    $progname =~ s|.*/([^/]+)$|$1|;    # on linux we have to strip the path and the extension
    $progname =~ s|\.[^\.]*$||;    # on linux we have to strip the path and the extension
    #$progname = File::Basename::basename($progname, '\..*');    # on linux we have to strip the path and the extension
	my $file = "/tmp/$progname-";
    while(1)
	  {
    	$file .= int(rand 10000) . $extension;
		if (!-e $file) { last; }
	  }
    return $file;
  }


# Returns true if the given rpm file is already installed at this version or higher.
sub isInstalled
  {
	my ($class, $rpmfile) = @_;
	my $out = GuiUtils->runcmd("LANG=C /bin/rpm -U --test $rpmfile", 2);
	# The rc is not reliable in this case because it will be 1 if it is already installed
	# of if there is some other problem like a dependency is not satisfied.  So we parse the
	# output instead.
	if (grep(/package .* already installed/, @$out)) { return 1; }
	else { return 0; }
  }


# Return the installed version #s of the given list of RPMs.  The results are returned in a
# hash with the RPM name as the key and the version string as the value.  If mode (the 1st arg)
# is 0, for RPMs that are not installed on the system, there will be no associated key or value.
# If mode=1, then for RPMs that are not installed on the system, the key will be the RPM name
# and the value will be "not installed".
sub getInstalledVersions
  {
	shift @_;
	my $mode = shift @_;
	my $out = GuiUtils->runcmd("LANG=C $::RPMCMD -q @_", 2);
	# Do not bother checking the rc because an rpm not installed will cause the rc to be 1
	my %hash;
	foreach my $l (@$out)
	  {
		my $notInstalled;
		if (($notInstalled) = $l=~/package\s+(\S+)\s+is not installed/)
		  {
			if ($mode == 1) { $hash{$notInstalled} = 'not installed'; }
			next;
		  }
		my ($rpm, $version) = split(/-/, $l, 2);
		if (defined($rpm) && defined($version)) { $hash{$rpm} = $version; }
	  }
	return %hash;
  }


# Return the latest file name that is returned by the given wildcard.  (Latest is
# determined by version numbers in the file names, not by modification times.)
# If no file if found, returns undef.
sub getLatestFile
  {
	my ($class, $file) = @_;
	if ($file=~/\.rpm$/ && -x '/usr/sbin/autoupdate')         # let autoupdate figure out versions
	  {
		my @files = NodeUtils->runcmd("/usr/sbin/autoupdate --showlatest $file", -1);
		if ($::RUNCMD_RC || !scalar(@files) || $files[0]=~/^Error:\s*No such file:/) { return undef; }
		else { return $files[0]; }
	  }
	else
	  {
		my @files = NodeUtils->runcmd("/bin/ls -vr $file", -1);
		if ($::RUNCMD_RC || !scalar(@files)) { return undef; }
		else { return $files[0]; }
	  }
  }


# Initialize a webmin page.  Call this at the very beginning of each page.
# It gets the input parameters and sets some global variables.
sub init
  {
	$::|=1;      # auto flush the output

	# Get the config values and provide defaults
	&::init_config();
	if (!length($::config{MonitorInterval})) { $::config{MonitorInterval} = 15; }
	if (!length($::config{MaxNodesDisplayed})) { $::config{MaxNodesDisplayed} = 50; }
	$::GUI_VERBOSE = $::config{GUI_VERBOSE};
	
	&::ReadParse();      # get the CGI form parameters
  }


# Create standard CSM page header.  Call this before producing any html output.
# Help (the 1st arg) is the name of the help page that should be associated with this page.
# CurrentTopic is an integer (starting at zero) indicating which main topic in the header that
# this page is part of.  If subTopic is specified, this will be the text that is shown right
# below the current topic.  If long (the 4th arg) is set to 1, the theme will be instructed to
# not use a table to format the page so output will appear as it is being produced.
sub addHeader
  {
	my ($class, $help, $currentTopic, $subTopic, $long) = @_;
	
	# Need to set this to make unbuffered i/o work since an unfinished table may cause browsers
	# to not display output as it is being produced.  But it makes the fancier themes not look as
	# good, so it should only be set if you need the output to display as it is produced.
	if ($long) { $::theme_no_table = 1; }
	
	# Determine what the background should be for this theme
	my $bodybg = '';
	#if ($::tb eq 'bgcolor=#9999ff')
	if ($::gconfig{theme}!~/^mscstyle/i)       # set the page background for the old and caldera themes
	  {
		#my $bg = '#fffff2';
		my $bg = '#ffffef';
		#my $bg = '#ffffc1';
		#my $bg = '#f4f4f4';
		$bodybg = "<style>BODY{background-color : $bg;}</style>\n";
	  }
	
	# Create title and header area
	&main::header('Cluster Systems Management', "$::CSMDIR/images/csmlogo.gif", $help, 1, 1, undef,
				  qq(<A HREF="$::CSMDIR/suggestions.html">Send suggestions for CSM web interface</A>),
				  qq(<link rel="stylesheet" type="text/css" href="$::CSMDIR/style.css">\n$bodybg\n) );
	# not sure what this cookie was for?		  qq(<link rel="stylesheet" type="text/css" href="$::CSMDIR/style.css">\n$cookie\n) );
	#print "tb=$::tb, cb=$::cb\n";

	# Create the main links (topics) at the top
	my @topics = ('CSM Home', 'Define Nodes', 'Edit Groups', 'Monitor',
				  'Config File Mgmt', 'Software Maint', 'Diagnose MS', 'CSM Config');
	my @tooltips = ('Main page of CSM Web Interface', 'Define new nodes to be added to the cluster', 'Create/change/delete node groups', 'Monitor the cluster with RMC conditions and responses',
				  'Manage configuration files on the nodes', 'Download and install software on the mgmt svr and nodes', 'Diagnose problems on the mgmt svr', 'View/change global CSM settings');
	my @links = ("$::CSMDIR/.", "$::CSMDIR/definenode.cgi", "$::CSMDIR/nodegrp.cgi", "$::CSMDIR/conditions", "$::CSMDIR/cfm.cgi", "$::CSMDIR/softmaint", "$::CSMDIR/diagms", "$::CSMDIR/config");
	print qq(<BR> <TABLE cellpadding="2" cellspacing="2"> <TBODY> <TR>\n);
	my $focusColor = '#999999';
	my $blurColor = '#cccccc';
	for (my $i=0; $i<scalar(@topics); $i++)
	  {
		if ($i == $currentTopic)
		  {
			if (length($subTopic)) { print qq(<TD bgcolor="$focusColor" align="center"><B><A href="$links[$i]" title="$tooltips[$i]">$topics[$i]</A></B></TD>\n); }
			else { print qq(<TD bgcolor="$focusColor" align="center"><B>$topics[$i]</B></TD>\n); }
		  }
		else { print qq(<TD bgcolor="$blurColor" align="center"><B><A href="$links[$i]" title="$tooltips[$i]">$topics[$i]</A></B></TD>\n); }
	  }
    if (length($subTopic))
	  {
		print qq(</TR><TR>\n);
		print '<TD></TD>'x$currentTopic;
		print qq(<TD bgcolor="$focusColor" align="center"><B><FONT size="-1">$subTopic</FONT></B></TD>\n);
	  }

	# Have to set this after the call to header().  That function seems to reset it?
	$::|=1;      # auto flush the output
    print qq(</TR> </TBODY> </TABLE> <BR>\n);
  }


# Convenience function for a page that has not been worked on yet.
sub notImplemented { GuiUtils->msg('I', '----- This page is not yet implemented. -----'); }


# Display messages in the html.  If severity is W or E, it will attempt to use the Error class
# from the style sheet.  If severity is V, it will only display if GUI_VERBOSE is set.
sub msg
  {
	my ($class, $severity, $msg) = @_;
	if ($severity=~/V/ && !$::GUI_VERBOSE) { return; }
	my $styleclass = 'Info';
	if ($severity =~ /[WE]/) { $styleclass = 'Error'; }
	print "<P class=$styleclass>$msg</P>\n";
  }


# Run a cmd in a suitable way for an HTML context.
# The cmd and output are handled differently depending on the mode argument:
#  0: If successful, return output in an array reference with the newlines removed.  Otherwise,
#     print the error msg with <P></P> and return rc in $::RUNCMD_RC.
#  1: This will be treated as a long running cmd, in which the cmd will be run with open()
#     and the output will be printed using html as it is being produced by the cmd.  Error
#     msgs will also be printed.
#  2: This will behave like mode=0 (run cmd and return output) except that no error msgs
#     will be displayed, but $::RUNCMD_RC will be set.
#  3: Behaves similarly to mode=1 (initiate a long running cmd), but instead of reading and
#     displaying the output, it returns the filehandle so the caller can read and process
#     the output.  It will also return any open errors in $::RUNCMD_RC, but the caller is
#     responsible for closing the filehandle and checking the rc (which will be the rc of the cmd).
# The 3rd arg is a reference to an options hash, e.g. { NoVerbose => 1, NoRedirectStderr => 1 }
sub runcmd
  {
	my ($class, $cmd, $mode, $options) = @_;
	$::RUNCMD_RC = 0;
	if (!($cmd =~ /2>&1$/) && !$$options{NoRedirectStderr}) { $cmd .= ' 2>&1'; }
	if (!$$options{NoVerbose} && $mode!=2 && $::config{VERBOSE}) { $cmd =~ s/^(\S+)/$1 -v/; }
	my ($MSGCAT, $MSGMAPPATH, $MSGSET) = ('nodecmds.cat', "$GuiUtils::csmroot/msgmaps", 'NodeUtils');
	if ($::GUI_VERBOSE)
	  {
		my $msg = NodeUtils->getMessageFromCat($MSGCAT, $MSGMAPPATH, $MSGSET, 'IMsgCMD', $cmd);
		GuiUtils->msg('I', $msg);
	  }
	my ($outref, $rc);
	if ($mode == 1 || $mode == 3) { $rc = !open(LONGCMD, "$cmd |"); }
	else { $outref = [];   @$outref = `$cmd`;  $rc = $? >> 8; }          # mode 0 or 2

	#todo: this check does not catch errors from the cmd itself for the long running cmd, because the cmd has not finished yet.
	#      After the output is done and we close the file, if we find the rc is bad
	#      we need to output javascript that will change the error msg output (that has already been printed) to the Error class.
	$::RUNCMD_RC = $rc;
	if ($rc && $mode!=2)
	  {
		my $errmsg = '';
		if (NodeUtils->isLinux() && $::RUNCMD_RC == 139) { $errmsg = "Segmentation fault  $errmsg"; }
		elsif ($mode == 1) { $errmsg = $!; }
		else
		  {
			NodeUtils->filterRmcApiOutput($cmd, $outref); # The error msgs from the -api cmds are pretty messy.  Clean them up a little.
			$errmsg = join('', @$outref);
			chomp $errmsg;
		  }
		$errmsg = NodeUtils->getMessageFromCat($MSGCAT, $MSGMAPPATH, $MSGSET, 'EMsgCMD_FAILED', 'GuiUtils::runcmd', $::RUNCMD_RC, $cmd, $errmsg);
		GuiUtils->msg('E', $errmsg);
		if ($mode == 1) { close(LONGCMD); }
		return;
	  }
	
	if ($mode == 1)    # long running cmd - read and display output
	  {
		$::|=1;        # set autoflush again just to be sure
		#print "<p>Set autoflush</p>\n";
		while(<LONGCMD>)
		  { print "$_<BR>\n"; }
		  #{ print "<p>$_</p>\n"; }
		
		if (!close(LONGCMD))
		  {
			$::RUNCMD_RC = $? >> 8;       # usually this is the rc of the cmd
			#GuiUtils->msg('E', "Exit code $::RUNCMD_RC from command $cmd.  Error message from cmd: $!.");
			my $errmsg = NodeUtils->getMessageFromCat($MSGCAT, $MSGMAPPATH, $MSGSET, 'EMsgCMD_FAILED', 'GuiUtils::runcmd', $::RUNCMD_RC, $cmd, (length($!) ? $! : '(see error message displayed above)'));
			GuiUtils->msg('E', $errmsg);
		  }
	  }
	elsif ($mode == 3) { return *LONGCMD; }
	else   # mode = 0 or 2
	  {
		chomp(@$outref);
		return $outref;
	  }
  }

# Process an HTML file that may have perl code in it too.
# Input:  location of file to use, and whether to skip interpolation of the html or not
sub processHTML
  {
	my ($class, $file, $noInterpolation) = @_;

	if ($file !~ /^\s*\//) { $file = "$ENV{PWD}/$file"; }
	open(HTML, "<$file") or die "cannot open file $file: $? $!";

	my $interpStr = '"';
	if ($noInterpolation) { $interpStr = "'"; }

	my @code = <HTML>;
	my $code = join('', @code);

	$code =~ s/^.*?<\/HEAD>\s*?<BODY.*?>//si;        # remove the header
	$code =~ s/<\/BODY>\s*?<\/HTML>\s*?$//si;        # remove the footer
	#$code =~ s/(<SCRIPT.*)\\(.*<\/SCRIPT>)/$1\\\\$2/sgi;    # escape backslashes in javascript

	my $i = 1;
	$code =~ s/<!--CSM/"\nEND_OF_HTML" . $i++ . "\n"/sige;
	$code .= "\nEND_OF_HTML$i\n";

	$code = "print <<${interpStr}END_OF_HTML1${interpStr};\n" . $code;
	$i = 2;
	$code =~ s/CSM-->/"\nprint <<${interpStr}END_OF_HTML" . $i++ . "${interpStr};\n"/sige;

	return $code;
  }


# Call this from your cgi file when you want it to act as a hidden refresh frame for
# another page that needs its display updated dynamically.  After calling this, run
# your commands to get the events and put them into the resstates hash before calling
# addRefreshBody().
sub addRefreshHeader
  {
	&main::PrintHeader();

	print <<"END_OF_HTML0";
<HTML><HEAD>
<META http-equiv="Content-Style-Type" content="text/css">
<META HTTP-EQUIV="REFRESH" CONTENT="$::config{MonitorInterval}">
<STYLE type="text/css"><!--P{font-size:8pt;}--></STYLE>
</HEAD><BODY>
END_OF_HTML0
  }


# Call this from your cgi file when you want it to act as a hidden refresh frame for
# another page that needs its display updated dynamically.  This should be called after run
# addRefreshHeader() and you run your commands to get the events and put them into the
# resstates hash.
sub addRefreshBody
  {
	my ($class, $app, $resStates, $checkedTime, $eventTime) = @_;
	#foreach my $k (keys %$resStates) { print "<p>$k=$$resStates{$k}</p>"; }

	if ($::GUI_VERBOSE) { print "<p><b>$app: Checked for events:</b><p>", scalar(localtime($checkedTime)), '<p><p><b>Last event received:</b><p>', scalar(localtime($eventTime)), "</p>\n"; }

	if (scalar(keys %$resStates))
	  {
		print <<"END_OF_HTML";
<SCRIPT language="JavaScript">
if (top && top.$app && top.$app.document && top.$app.loadingDone)
{
var doc = top.$app.document;
var a = new Array();
END_OF_HTML

		foreach my $k (keys %$resStates)
		  {
			if ($::GUI_VERBOSE) { print "document.write('<p>$k = $$resStates{$k}</p>');\n"; }
			my $value = $$resStates{$k};
			if (!($value=~/^(\d+\.?\d*|\.\d+)$/)) { $value = "'$value'"; }    # if it is not a number, surround it with quotes
			print "a.push({key:'$k', value:$value});\n";
		  }

		print <<"END_OF_HTML2";
top.$app.updateData(a);
}
else { document.write('<p>Data frame not fully loaded yet.  Waiting...</p>'); }
</SCRIPT>
END_OF_HTML2

	  }    # if (scalar(keys %$resStates))

	print "</BODY></HTML>\n";
  }


# Initialize our tracking of events.  This must be called before getEvents().
# EventFilename is optional and is a filename that should be used to track the last time
# events were queried from the audit log.
sub initEvents
  {
	my ($class, $eventFilename) = @_;

	my $LAST_EVENT_FILE = length($eventFilename) ? $eventFilename : '.LastEventTime';
	if (!(-e $LAST_EVENT_FILE)) { open TMP, ">>$LAST_EVENT_FILE";  close TMP; }   # create the file if it does not exist
	
	# Just need to get the last sequence # in the AuditLog and store it in the file
	my ($time, $sequenceNumber) = GuiUtils->getLastTimeAndSeqNum();
	if (defined($time)) { GuiUtils->storeTimeAndSeqNum($time, $sequenceNumber, $LAST_EVENT_FILE); }

	return;
}


# Store the most recent event time and sequence num in the mod times of the file given.
# Returns the time in a localized, number of seconds form for reporting purposes.
sub storeTimeAndSeqNum
  {
	my ($class, $time, $sequenceNumber, $filename) = @_;
	my ($mon, $d, $y, $h, $min, $s) = $time=~m|^(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):(\d+)|;
	my $t = main::timelocal($s, $min, $h, $d, $mon-1, $y);
	utime($t, hex($sequenceNumber), $filename);
	return $t;
  }


# Get the most recent event time and sequence num from the mod times of the file given.
sub retrieveTimeAndSeqNum
  {
	my ($class, $filename) = @_;
	my ($j1, $j2, $j3, $j4, $j5, $j6, $j7, $j8, $atime, $mtime, $j9, $j10, $j11) = stat $filename;
	my $sequenceNumber = sprintf('0x%lx', $mtime);
	my $eventTime = $atime;
	return ($eventTime, $sequenceNumber);
  }


# Get last sequence # in the auditlog so future queries to auditlog can start from this point.
# If onlyFastMethod is 1, then only try the time range focused call, and if that does not work just
# return 0 for the seq # and time.
sub getLastTimeAndSeqNum
  {
	my ($class, $onlyFastMethod) = @_;
	# For performance, 1st try just todays entries instead of listing the whole audit log
	my $out = GuiUtils->runcmd(qq(LANG=C /usr/bin/lsaudrec -x -S ERRM -s "Time >#-0001" SequenceNumber Time | tail -n 1), 0, {NoVerbose=>1});
	if (!scalar(@$out))
	  {
		if ($onlyFastMethod) { return (0, 0); }
		# Did not get any entries so list the whole audit log
		$out = GuiUtils->runcmd(qq(LANG=C /usr/bin/lsaudrec -x -S ERRM -s "TemplateId=38" SequenceNumber Time | tail -n 1), 0, {NoVerbose=>1});
	  }
	
	my ($sequenceNumber, $time) = split(/\s+/, $$out[0], 2);
	return ($time, $sequenceNumber);
  }

# This function set the attribut value of $a to $values (a reference of hash) 
# from the $hohInfo (reference of hash). The values in $value is used to
# display to user on GUI. 
sub setValues {
	shift;
	my $a = shift;
	my $values = shift;
	my $hohInfo = shift;

	my $mixedValueFlag = 0;
	my $attrValue;
	my $mixedValue = "Mixed Values: ";
	my $count = 0;
	foreach my $hostname (keys %{$$hohInfo{$a}}) {
		if ($count != 0 && 
			$attrValue ne $$hohInfo{$a}{$hostname}) {
			$mixedValueFlag = 1;
		}
		$count += 1;
		$attrValue = $$hohInfo{$a}{$hostname};
		$mixedValue .= "$hostname=$$hohInfo{$a}{$hostname}, ";
	} 
	if ($mixedValueFlag) {
		$$values{$a} = $mixedValue;
		$$values{$a} =~ s/, $//;
	} else {
		$$values{$a} = $attrValue;
	}
}

# This function parse the values in the $values for attribute $attr. The 
# parsed value is returned through a hash %attrHostValue.
sub parseValues {
	shift;
	my $attr = shift;
	my $values = shift;
	my $node_array = shift;

	my $attrValue = $$values{$attr};
	chomp($attrValue);
	my %attrHostValue = ();
	
	if ($attrValue =~ m/^\s*Mixed\s+Values\s*:\s*/) {
		$attrValue =~ s/^\s*Mixed\s+Values\s*:\s*//;
		my @v = split(/\s*,\s*/, $attrValue);
		foreach my $ha (@v) {
			my ($host, $a) = split(/\s*=\s*/, $ha);
			$attrHostValue{$host} = $a;
		}
	} else {
		if (scalar(@$node_array)) {
			foreach my $host (@$node_array) {
				$attrHostValue{$host} = $attrValue;
			}
		} else {
			$attrHostValue{"no_hostname"} = $attrValue;
		}
	}
	return (\%attrHostValue);
}

sub isEmpty {
	shift;
	my $val = shift;
	if ($val =~ m/^\s*$/) {
		return 1;
	} else {
		return 0;
	}
}

# This function get the Hardware Information from the HPC. It obtains the 
# PowerMethod and HWControlPoint from the $values (reference of hash), then
# use this information to query the HPC. The Hardware Information is returned
# through the $info array reference.
sub getHcpHwInfo {
	shift;
	my $values = shift;
	my $node_array = shift;

	my $rc = 0;

	my $powerMethod = GuiUtils->parseValues("PowerMethod", $values, $node_array);
	my $hwControlPoint = GuiUtils->parseValues("HWControlPoint", $values, $node_array);
	my $info = [];
	my @cmd = ();
	if (scalar(@$node_array)) {
		foreach my $host (@$node_array) {
			my $command;

			if (!GuiUtils->isEmpty($$powerMethod{$host}) && 
				!GuiUtils->isEmpty($$hwControlPoint{$host})) {
				$command = "/opt/csm/bin/lshwinfo -p $$powerMethod{$host} -c $$hwControlPoint{$host}";
			} else {
				$rc = 1;
			}

			if (!GuiUtils->isEmpty($command)) {
				push @cmd, $command;
			}
		}
	} else {
		my $pm = $$powerMethod{'no_hostname'};
		my $hc = $$hwControlPoint{'no_hostname'};
		if ($pm ne '' && $hc ne '') {
			my $command = "/opt/csm/bin/lshwinfo -p $pm -c $hc";
			push @cmd, $command;
		}
	}

	if (scalar(@cmd) == 0) { $rc = 1; }
	my @sortedCmd = sort @cmd;
	my $lastCmd;
	foreach my $command (@sortedCmd) {
		if ($lastCmd ne $command) {
			my $hwinfo = GuiUtils->runcmd("LANG=C $command", 3);



			while (my $line = <$hwinfo>) {
				if ($line =~ m/^\S+::\S+::\S+::\S+::\S*::\S*::\S*::\S*/) {
					push @$info, $line;
					#print(qq(<p>Line: $line</p>));
				} elsif ($::RUNCMD_RC) {
					GuiUtils->msg('E', $line);
					$rc = 2;
				} else {
					if (!($line =~ m/^\s*#/)) {
						GuiUtils->msg('V', $line);
					}
				}
			}
			close($hwinfo);

		}
		$lastCmd = $command;

		#foreach my $content (@$info) {
		#	print(qq(<p>info: $content </p>));
		#}
	}

	return ($rc, $info);
}

# This function parse the hardware information in the $info (reference of array)
# into the $hohInfo (reference of a hash).
sub parseHCPHwInfo {
	shift;
	my $info = shift;
	my $node_array = shift;
	my $hohInfo = shift;

	foreach my $i (0 .. $#$info) {
		if ($$info[$i] =~ m/^\s*#/) {next;}
		if ($$info[$i] =~ m/^\s*$/) {next;}
		my @attr = split /\:\:/, $$info[$i];
		if (scalar(@$node_array)) {
			foreach my $node (@$node_array) {
				if ($node eq $attr[0]) {
					$$hohInfo{PowerMethod}{$node} = $attr[1];
					$$hohInfo{HWControlPoint}{$node} = $attr[2];
					$$hohInfo{HWControlNodeId}{$node} = $attr[3];
					$$hohInfo{LParID}{$node} = $attr[4];
					$$hohInfo{HWType}{$node} = $attr[5];
					$$hohInfo{HWModel}{$node} = $attr[6];
					$$hohInfo{HWSerialNum}{$node} = $attr[7];
					last;
				}
			}
		} else {
			my $node = sprintf("%05d_%s", $i,$attr[0]);
			$$hohInfo{PowerMethod}{$node} = $attr[1];
			$$hohInfo{HWControlPoint}{$node} = $attr[2];
			$$hohInfo{HWControlNodeId}{$node} = $attr[3];
			$$hohInfo{LParID}{$node} = $attr[4];
			$$hohInfo{HWType}{$node} = $attr[5];
			$$hohInfo{HWModel}{$node} = $attr[6];
			$$hohInfo{HWSerialNum}{$node} = $attr[7];
				
		}
	}
}

# This function reverse the hash hash.
# Input is a reference of hash of hash.
# Output is a reference of reversed hash of hash.
sub reverseHashHash {
	shift;
	my $original = shift;
	my %reverse = ();

	foreach my $a (keys %$original) {
		foreach my $b (keys %{$$original{$a}}) {
			$reverse{$b}{$a} = $$original{$a}{$b};
		}
	}
	return (\%reverse);
}

# This function splits the original values and current values from the values 
# submitted by a html page. The %::in is a hash of the valuse submitted by 
# the html page. It includes the current values and original values. The 
# original valuse are the values last submission of a html page.
# The $valuses is a hash reference which will get the current values.
# The $orig is a hash reference which will grt the original values.

sub splitInputValues {
	shift;
	my $values = shift;
	my $orig = shift;

	# Put original values to %$orig, put input values to %$values
	foreach my $key (keys %::in) {
		if ($key =~ /^orig/) {
			#this attribute is an original attribute
			my $attr = $key;
			$attr =~ s/^orig//;
			$$orig{$attr} = $::in{$key};
			chomp($$orig{$attr});
			$$orig{$attr} =~ s/\s+$//;
		} else {
			$$values{$key} = $::in{$key};
			chomp($$values{$key});
			$$values{$key} =~ s/\s+$//;
		}
		delete $::in{$key};
	}
}

# This function will get all the current params in a hash (referenced by $p)
# and convert it to the url style parameters. The returned value can be used
# in an url as url parameters.
sub getParams {
	shift;
	my $p = shift;

	my @params;
	foreach my $key (keys %$p) {
		#&::urlize("");
		push @params, "$key=" . &::urlize($$p{$key});
	}
	return join('&', @params);
}

# This function outputs a small left-facing arrow button followed by text 
# "$actionName". Any CGI program that calls addHeader must also call this 
# function at the end in order to complete the page's HTML properly. The big 
# difference between this function and the footer() function, which came with 
# the webmin library, is that the addFooter() uses form and POST to submit 
# actions and variables and the footer() uses hyperlinks.
#
# This function takes the following input
#  1. $action, a string that contents the action URL.
#  2. $actionName, a string that contents the action name.
#  3. $varref, a hash reference of variables that need to submit when the 
#       button is click by suer.
# Note, this functions works on Old Webmin Theme, Calderal Theme and 
# MSC.Linux Theme. 

sub addFooter {
	shift;
	my $action = shift;
	my $actionName = shift;
	my $varref = shift;

	my $imgsrc="/images/left.gif";
	my $color='#0000ff';

	# check the current theme
	if ($::gconfig{theme} =~ m/^mscstyle/) {
		if ($::theme_no_table) {
			print "</UL>\n";
		} elsif (@::header_arguments > 1 && !$::header_arguments[5]) {
			print(qq(</TABLE></TABLE><BR>\n));
		}

		$imgsrc="/images/arrow.jpg";
		$color='#000000';
	}
	#print (qq( <p> theme = $::gconfig{theme} </p>));
	#print (qq( <p> postbody = $::gconfig{postbody} </p>));
	#print (qq( <p> postbodyinclude = $::root_directory/$::current_theme/$::tconfig{'postbodyinclude'} </p>));
	#print (qq(<p> Theme: $::root_directory/$::current_theme/$::tconfig{'functions'}</p>));

	# post form
	print(qq(<FORM method="POST" target="_top" action="$action" name="returnForm">\n));
	print(qq(<TABLE>\n));
	print(qq(  <TBODY>\n));
	print(qq(    <TR>\n));
	print(qq(      <TD><INPUT type="image" name="ReturnButton" value="Return" src="$imgsrc" alt="<-"></TD>\n));
	print(qq(      <TD><FONT color="$color">Return to $actionName</FONT></TD>\n));
	print(qq(    </TR>\n));
	print(qq(  </TBODY>\n));
	print(qq(</TABLE>\n));
	foreach my $var (keys %$varref) {
		print(qq(<INPUT type="hidden" name="$var" value="$$varref{$var}">\n));
	}
	print(qq(</FORM>\n));
	print "<BR>\n";

	# post body
	my $postbody = $::tconfig{'postbody'};
	#$postbody = "%HOSTNAME%, %VERSION%, %USER%, %OS%"; # for test postbody
	if ($postbody) {
		my $hostname = &::get_system_hostname();
		my $version = &::get_webmin_version();
		my $os_type = $::gconfig{'real_os_type'} ?
				$::gconfig{'real_os_type'} : $::gconfig{'os_type'};
		my $os_version = $::gconfig{'real_os_version'} ?
				$::gconfig{'real_os_version'} : $::gconfig{'os_version'};
		$postbody =~ s/%HOSTNAME%/$hostname/g;
		$postbody =~ s/%VERSION%/$version/g;
		$postbody =~ s/%USER%/$::remote_user/g;
		$postbody =~ s/%OS%/$os_type $os_version/g;
		print "$postbody\n";
		}
	if ($::tconfig{'postbodyinclude'}) {
		open(INC,
		 "$::root_directory/$::current_theme/$::tconfig{'postbodyinclude'}");
		while(<INC>) {
			print;
			}
		close(INC);
		}
	if (defined(&theme_postbody)) {
		&theme_postbody(@_);
		}
	print(qq(</BODY>\n));
	print(qq(</HTML>\n));
}

# Get nodes attributes from the CSM Database.
# Input $node_array is a reference of a node array, input $what is the list of
# node attributes that try to get. Here is the format of this list
# "Hostname::PowerMethod::HWControlPoint::HWControlNodeId".
# The output is a reference of hash-hash in form of ${attribute}{node}.

sub getNodeAttrFromCSMDB {
	shift;
	my $node_array = shift;
	my $what = shift;
	my $where = NodeUtils->buildWhereInList($node_array, 'Hostname');
#	my $what = "Hostname::PowerMethod::HWControlPoint::HWControlNodeId::LParID::HWType::HWModel::HWSerialNum::InstallAdapterMacaddr::PowerStatus::Status::InstallDistributionName::InstallDistributionVersion::InstallMethod";
	my $exist_attr = GuiUtils->runcmd("LANG=C /usr/bin/lsrsrc-api -D ':|:' -n -i -s IBM.ManagedNode::${where}::${what}", 3);

	my %hohData = ();

	while (my $line = <$exist_attr>) {
		if ($line =~ m/^Hostname:|:\s+:|:PowerMethod:|:\s+:|:\S+/) {
			my %attrs = split(/:\|:/, $line);
			my $hostname = $attrs{Hostname};
			delete $attrs{Hostname};
			foreach my $a (keys %attrs) {
				$hohData{$a}{$hostname} = $attrs{$a};
				delete $attrs{$a};
			}
		} elsif ($::RUNCMD_RC) {
			GuiUtils->msg('E', $line);
		} else {
			GuiUtils->msg('V', $line);
		}
	}
	close($exist_attr);

	return(\%hohData);
}

1;
