#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display the RMC Audit Log

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;

GuiUtils->addHeader('auditlog', 3, 'Audit Log');

my $code = GuiUtils->processHTML('auditlog-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

if (exists($::in{displayButton}))
  {
	my $cmd = '/usr/bin/lsaudrec -S ERRM';
	if (exists($::in{longCheckbox})) { $cmd .= ' -l'; }

	# Process all the fields that need to go in the select string
	my $selectstr;
	if ($::in{minuteField}=~/\S/ || $::in{hourField}=~/\S/ || $::in{dayField}=~/\S/ || $::in{monthField}=~/\S/ || $::in{yearField}=~/\S/)
	  {
		# The format of the time string is:  #-mmddhhmmyyyy
		$selectstr = "Time >#";
		if (exists($::in{relativeTimeCheckbox})) { $selectstr .= '-'; }
		$selectstr .= twoDigits($::in{monthField});
		$selectstr .= twoDigits($::in{dayField});
		$selectstr .= twoDigits($::in{hourField});
		$selectstr .= twoDigits($::in{minuteField});
		$selectstr .= (($::in{yearField}<90&&!exists($::in{relativeTimeCheckbox})) ? 2000+$::in{yearField} : $::in{yearField});
	  }
	if (length($::in{conditionField})) { my $and = $selectstr?' &&':'';  $selectstr .= "$and ConditionName='$::in{conditionField}'"; }
	if (length($::in{responseField})) { my $and = $selectstr?' &&':'';  $selectstr .= "$and EventResponseName='$::in{responseField}'"; }
	if (exists($::in{onlyEventsCheckbox}) && exists($::in{onlyResponsesCheckbox})) { my $and = $selectstr?' &&':'';  $selectstr .= "$and (TemplateId=38 || TemplateId>=39 && TemplateId<=51)"; }
	elsif (exists($::in{onlyEventsCheckbox})) { my $and = $selectstr?' &&':'';  $selectstr .= "$and TemplateId=38"; }
	elsif (exists($::in{onlyResponsesCheckbox})) { my $and = $selectstr?' &&':'';  $selectstr .= "$and TemplateId>=39 && TemplateId<=51"; }
	if (length($::in{maxEntriesField}) && !length($selectstr))
	  {
		# Try to get the last sequence # so we can list only the right number of entries
		my ($time, $seqnum) = GuiUtils->getLastTimeAndSeqNum(1);
		if ($seqnum)
		  {
			$seqnum -= $::in{maxEntriesField};
			if ($seqnum < 0) { $seqnum = 0; }
			$selectstr = "SequenceNumber>$seqnum";
		  }
		
		# 1st try a more focussed time range for performance reasons
		#$origselectstr = $selectstr;
		#my $and = $selectstr?'&&':'';
		#my $numdays = int($::in{maxEntriesField} / 20);        # estimate 20 entries per day
		#$selectstr .= "$and Time >#-$numdays";
	  }
	if (length($selectstr)) { $cmd .= qq( -s "$selectstr"); }

	if (length($::in{maxEntriesField})) { $cmd .= " | tail -n $::in{maxEntriesField}"; }
	GuiUtils->runcmd($cmd, 1, {NoVerbose=>1});
  }

&footer('.', 'Monitor');


sub twoDigits
  {
	my $v = $_[0];
	$v += 0;
	return ('0' x ($v<=9)) . $v;
  }
