#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display properties for a conditions

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;

## put in ACL checks here if needed

my $create = exists($::in{createButton}) || $::in{changeButton} eq 'Create';     # whether we are creating a new condition or showing properties of an existing condition
my $doit = exists($::in{changeButton});     # this means they pressed the Change or Create button from this page

GuiUtils->addHeader('cond-props', 3, $create?'Create Condition':'Condition Properties');

#my ($name, $nodeNames, $managementScope, $resourceClass, $selectionString, $eventExpression, $rearmExpression, $monitorStatus, $eventOccurred, $severity, $immediateEvaluate);
my %values;
my ($condition, $changedAttrStr);

if ($doit)           # creating a condition or changing properties
  {
	if ($create)        # create condition
	  {
		my $cmd = qq(mkrsrc-api IBM.Condition);
		foreach my $k (keys %::in)
		  {
			my $a;
			if ((($a)=$k=~/^input(.+)$/) && length($::in{$k}))        # if it is an input field and non-blank
			  {
				my $v = $::in{$k};     # the value
				if ($a eq 'NodeNames' && $v!~/^\{.*\}$/) { $v = "{$v}"; }
				$cmd .= "::${a}::" . NodeUtils->quote($v);
			  }
		  }
		my $outref = GuiUtils->runcmd($cmd);
		$condition = $::in{inputName};
		if (!$::RUNCMD_RC) { $changedAttrStr = "Condition $condition created."; }
	  }
	else      # change properties
	  {
		# Get original values
		my %orig = split(/:\|:/, $::in{origValues});
		#print "<p>orig=", join(',',%orig), "</p>\n";

		my %changedAttrs;
		foreach my $k (keys %::in)
		  {
			my $a;
			if ((($a)=$k=~/^input(.+)$/) && ($::in{$k} ne $orig{$a}))       # if it is an input field and different from the original value
			  {
				my $v = $::in{$k};     # the value
				if ($a eq 'NodeNames' && $v!~/^\{.*\}$/) { $v = "{$v}"; }
				$changedAttrs{$a} = NodeUtils->quote($v);
			  }
		  }
		$condition = $orig{Name};
		if (scalar(%changedAttrs))
		  {
			#if (exists($changedAttrs{NodeNames}) && $changedAttrs{NodeNames}!~/^\{.*\}$/) { $changedAttrs{NodeNames} = '{' . $changedAttrs{NodeNames} . '}'; }
			my $attrStr = join('::', %changedAttrs);
			my $outref = GuiUtils->runcmd(qq(chrsrc-api -s IBM.Condition::"Name='$condition'"::$attrStr));
			if (!$::RUNCMD_RC)
			  {
				$condition = $::in{inputName};           # in case they changed it
				$changedAttrStr = 'Properties changed: ' . join(', ', keys %changedAttrs);
			  }
		  }
		else { $changedAttrStr = 'No properties were changed.'; }
	  }
  }
else     # only displaying
  {
	$condition = $::in{conditions};
  }

# Whether we are 1st coming to this page or have changed/created a condition, display it
my @eventOccurred;
my @monitorStatus;
my $monitorStatusBin;
my %showEvent = ();
my %showStatus = ();

my $uncheckedBoxImg = "unchecked-box.gif";
my $checkedBoxImg = "checked-box.gif";
# Status;
my $msAssociatedImg = $uncheckedBoxImg;
my $msEventOccurredImg = $uncheckedBoxImg;
my $msEventErrorImg = $uncheckedBoxImg;
my $msScopeErrorImg = $uncheckedBoxImg;
# Event flags
my $efRearmImg = $uncheckedBoxImg;
my $efExprFalseImg = $uncheckedBoxImg;
my $efImmedEvalImg = $uncheckedBoxImg;
my $efRefreshImg = $uncheckedBoxImg;
my $efMissingPattrImg = $uncheckedBoxImg;
my $efUnassignImg = $uncheckedBoxImg;
my $efUnassignUndefImg = $uncheckedBoxImg;
my $efUnassignNoMatchImg = $uncheckedBoxImg;
my $efUnassignNogroupImg = $uncheckedBoxImg;
my $efQueryEventImg = $uncheckedBoxImg;
my $efAssignResourceImg = $uncheckedBoxImg;
my $efAssignNewResourceImg = $uncheckedBoxImg;

if (length($condition))
  {
	my $cmd = qq(lsrsrc-api -D ':|:' -i -n -s IBM.Condition::"Name IN ('$condition')"::Name::NodeNames::ManagementScope::ResourceClass::SelectionString::EventExpression::EventDescription::RearmExpression::RearmDescription::MonitorStatus::EventOccurred::Severity::ImmediateEvaluate);
	my $outref = GuiUtils->runcmd($cmd);
	
	%values = split(/:\|:/, $$outref[0]);      # this puts the pairs of attr name and value in the hash correctly
	# Get event occurred
	my $event = $values{EventOccurred};
	$event =~ s/^\[//;
	$event =~ s/\]$//;
	@eventOccurred = split(/,/, $event);

	# Process event for display
	$showEvent{EventOccurred} = $eventOccurred[0];
	$showEvent{ErrNum} = $eventOccurred[1];
	$showEvent{ErrMsg} = $eventOccurred[2];

	# Event flags
	my $ef = $eventOccurred[3];
	$efRearmImg = $checkedBoxImg if ($ef & 0x0001);
	$efExprFalseImg = $checkedBoxImg if ($ef & 0x0002);
	$efImmedEvalImg = $checkedBoxImg if ($ef & 0x0004);
	$efRefreshImg = $checkedBoxImg if ($ef & 0x0008);
	$efMissingPattrImg = $checkedBoxImg if ($ef & 0x0010);
	$efUnassignImg = $checkedBoxImg if ($ef & 0x0020);
	$efUnassignUndefImg = $checkedBoxImg if ($ef & 0x0040);
	$efUnassignNoMatchImg = $checkedBoxImg if ($ef & 0x0080);
	$efUnassignNogroupImg = $checkedBoxImg if ($ef & 0x0100);
	$efQueryEventImg = $checkedBoxImg if ($ef & 0x0200);
	$efAssignResourceImg = $checkedBoxImg if ($ef & 0x0400);
	$efAssignNewResourceImg = $checkedBoxImg if ($ef & 0x0800);


	if ($eventOccurred[4] != 0) {
		my $date = localtime ($eventOccurred[4]);
		my ($time, $year) = ($date =~ m/^(\S+\s+\S+\s+\S+\s+\S+)\s+(\S+)$/);
		$showEvent{EventTime} = "$time.$eventOccurred[5] $year";
	} else {
		$showEvent{EventTime} = "";
	}

	$showEvent{RsrcHndl} = $eventOccurred[6];

	my @ctDataType = ( #RSCT ct_data_type_t in ct.h
		"CT_UNKNOWN",
		"CT_NONE", # /*for Quantum variables only             */
		"CT_INT32",
		"CT_UINT32",
		"CT_INT64",
		"CT_UINT64",
		"CT_FLOAT32",
		"CT_FLOAT64",
		"CT_CHAR_PTR",   # /* pointer to NULL-terminated string     */
		"CT_BINARY_PTR",   #  /* pointer to ct_binary_t */
		"CT_RSRC_HANDLE_PTR",  # /* pointer to ct_resource_handle_t */
		"CT_SD_PTR",  #  /* pointer to ct_sd_t */
		"CT_SBS_PTR",   #  /* pointer to ct_sbs_t */
		"CT_INT32_ARRAY",  #  /* CT_INT32 array */
		"CT_UINT32_ARRAY",  # /* CT_UINT32 array */
		"CT_INT64_ARRAY",   #  /* CT_INT64 array */
		"CT_UINT64_ARRAY",   # /* CT_UINT64 array */
		"CT_FLOAT32_ARRAY",  # /* CT_FLOAT32 array */
		"CT_FLOAT64_ARRAY",  # /* CT_FLOAT64 array */
		"CT_CHAR_PTR_ARRAY", # /* CT_CHAR_PTR array */
		"CT_BINARY_PTR_ARRAY", # /* CT_BINARY_PTR array */
		"CT_RSRC_HANDLE_PTR_ARRAY",  # /* CT_RSRC_HANDLE_PTR array  */
		"CT_SD_PTR_ARRAY"            # /* CT_SD_PTR array */
	);
	$showEvent{DynAttrDataType} = $ctDataType[$eventOccurred[7]];

	$showEvent{RsrcName} = $eventOccurred[8];
	$showEvent{NodeName} = $eventOccurred[9];
	$showEvent{DynAttrValue} = $eventOccurred[10];

	# Get monitor status
	my $status = $values{MonitorStatus};
	$status =~ s/^\[//;
	$status =~ s/\]$//;
	@monitorStatus = split(/,/, $status);

	# Process status
	$showStatus{MonitorStatus} = $monitorStatus[0];
	if ($showStatus{MonitorStatus} & 0x0001) {
		$msAssociatedImg = $checkedBoxImg;
	}
	if ($showStatus{MonitorStatus} & 0x0002) {
		$msEventOccurredImg = $checkedBoxImg;
	}
	if ($showStatus{MonitorStatus} & 0x0004) {
		$msEventErrorImg = $checkedBoxImg;
	}
	if (($showStatus{MonitorStatus} & 0x0008) ||
		($showStatus{MonitorStatus} & 0x0010)) {
		$msScopeErrorImg = $checkedBoxImg;
	}
	$showStatus{NodeNames} = $monitorStatus[1];
	$showStatus{ErrCodes} = $monitorStatus[2];
	#$monitorStatusBin = sprintf("%032b", $monitorStatus[0]);
	
#	($name, $nodeNames, $managementScope, $resourceClass, $selectionString, $eventExpression, $rearmExpression, $monitorStatus, $eventOccurred, $severity, $immediateEvaluate) = split(/:\|:/, $$outref[0]);
	#$values{SelectionString} = NodeUtils->quote($values{SelectionString});
  }

my $pageTitle = $create ? 'Create a Condition' : "Properties for Condition: $condition";
my $pageDescription = $create ? 'Create a condition by filling in property values and clicking on Create.' : 'View or change the properties of this condition.';
my $changeButtonStr = $create ? 'Create' : 'Change';
my $nameDescription = $create ? 'Change this value when creating a new condition.' : 'Changing this will change the name of the existing condition (not create a new condition).';

my $code = GuiUtils->processHTML('cond-props-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('.', 'Monitor');

