#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display conditions and responses

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
my $LAST_EVENT_FILE = '.LastCondEventTime';     # the place to save the last sequence # we have read in the auditlog

# Go to the page corresponding to the button
if ($::in{refresh})      # act as the hidden refresh frame for the condition page
  {
	GuiUtils->addRefreshHeader();
	my ($resStates, $checkedTime, $eventTime) = conditions();
	GuiUtils->addRefreshBody('conditions', $resStates, $checkedTime, $eventTime);
	return;
  }
elsif ($::in{propButton}) { &redirect("cond-props.cgi?" . getParams()); }
elsif ($::in{createButton}) { &redirect("cond-props.cgi?" . getParams()); }
elsif ($::in{resPropButton} || $::in{resCreateButton}) { &redirect("resp-props.cgi?" . $ENV{QUERY_STRING}); }
elsif ($::in{auditButton}) { &redirect("auditlog.cgi?" . getParams()); }


GuiUtils->addHeader('conditions', 3);
#&main::header('Cluster Systems Management', 'images/csmlogo.gif', 'conditions', 1, 1, undef,
#			  '<A HREF="mailto:cluster@us.ibm.com?Subject=CSM Web Interface Suggestion">Send suggestions for CSM web interface</A>',
#			  '<link rel="stylesheet" type="text/css" href="style.css">' );

if ($::in{remButton})     # They are deleting some conditions
  {
	my @conditions;
	foreach my $k (keys %::in) { if ($k =~ /^condition\d+/) { push @conditions, $::in{$k}; } }
	my $where = NodeUtils->buildWhereInList(\@conditions, 'Name');
	GuiUtils->runcmd(qq(rmrsrc-api -s IBM.Condition::$where));
  }
elsif (exists($::in{changeAssoc}))      # they changed an association checkbox
  {
	my ($condition, $response);
	foreach my $k (keys %::in) { if (($condition) = $k=~/^assoc(.+)$/) { $response = $::in{$k}; last; } }

	# 1st check if it is already started/stopped
	my $outref = GuiUtils->runcmd("lscondresp -xd $condition $response");
	my $started = $$outref[0]=~/^.*:.*:.*:\"Active/;
	if ($started != $::in{changeAssoc})
	  {
		my $cmd = ($::in{changeAssoc} ? 'startcondresp' : 'stopcondresp');
		GuiUtils->runcmd("$cmd $condition $response");
	  }
  }
elsif ($::in{resRemButton})     # They are deleting a response
  {
	#todo:  maybe should handle the -f (force) option of rmresponse
	my $response = NodeUtils->quote($::in{responseList});
	GuiUtils->runcmd(qq(rmresponse $response));
  }
elsif ($::in{assocButton} || $::in{disassocButton})      # They are associating or disassociating a response with a condition
  {
	foreach my $k (keys %::in)
	  {
		if ($k =~ /^condition\d+/)
		  {
			my $cmd = $::in{assocButton} ? 'startcondresp ' : 'rmcondresp ';
			GuiUtils->runcmd($cmd . NodeUtils->quote($::in{$k}) . ' ' . NodeUtils->quote($::in{responseList}));
		  }
	  }
  }

GuiUtils->initEvents($LAST_EVENT_FILE);

#foreach my $k (keys %$resStates)
#  {
#	$$resStates{$k} = $$resStates{$k} ? 'images/red-ball-m.gif' : 'images/green-ball-m.gif';
#  }

#my $cmd = qq(lsrsrc-api -D ':|:' -i -s IBM.Condition::::Name::EventExpression::RearmExpression::MonitorStatus);
my $cmd = qq(lsrsrc-api -m -D ':|:' -i -s IBM.Condition::::Name::EventExpression::RearmExpression::MonitorStatus::ResourceHandle::EventOccurred -s IBM.EventResponse::::ResourceHandle::Name -s IBM.Association::::ConditionHandle::EventResponseHandle::ActiveFlag);
my $outref = GuiUtils->runcmd($cmd);

# Process the output
my (@conditions, %responses, %associations);
foreach my $line (@$outref)
  {
	my ($class, $rest) = split(/:\|:/, $line, 2);
	if ($class eq 'IBM.Condition') { push @conditions, $rest; }
	elsif ($class eq 'IBM.EventResponse')
	  {
		my ($reshandle, $name) = split(/:\|:/, $rest);
		$responses{$reshandle} = $name;
	  }
	elsif ($class eq 'IBM.Association')
	  {
		my ($condhandle, $resphandle, $active) = split(/:\|:/, $rest);
		$associations{$condhandle} .= "$resphandle,$active:|:";
		#print "$condhandle=$resphandle,$active:|:<BR>\n";
	  }
  }
@conditions = sort @conditions;
#my ($k, $v);  while (($k, $v) = each %associations) { print "$k=$v<BR>\n"; }

my $code = GuiUtils->processHTML('cond-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

#print <<'END_OF_HTML';
#<SCRIPT language="JavaScript">
#<!--
#  var condrefresh = window.open('cond-refresh.cgi', 'condrefresh', 'width=150,heigth=300,resizable=yes');
#//-->
#</SCRIPT>
#END_OF_HTML

&footer('', 'CSM Home');


sub getParams
  {
	my @conditions;
	my @params;
	foreach my $k (keys %::in)
	  {
		if ($k =~ /^condition\d+/) { push @conditions, &urlize($::in{$k}); }
		elsif ($k !~ /^status/) { push @params, "$k=" . &urlize($::in{$k}); }
	  }
	#push @params, 'nodes=' . &urlize(join(',', @nodes));  # is it necessary to urlize this?
	if (scalar(@conditions)) { push @params, 'conditions=' . join(',', @conditions); }
	return join('&', @params);
  }


sub conditions
  {
	my ($sequenceNumber, $eventTime, $time);
	my $resStates = {};
	
	# Get the time of the last event so we know when to check since
	($eventTime, $sequenceNumber) = GuiUtils->retrieveTimeAndSeqNum($LAST_EVENT_FILE);

	my $out = GuiUtils->runcmd(qq(LANG=C /usr/bin/lsaudrec -x -S ERRM -s "SequenceNumber>$sequenceNumber && TemplateId=38" Time Subsystem SequenceNumber Message), 0, {NoVerbose=>1});
	#print "out = ", scalar(@$out), "\n";
	foreach my $line (@$out)
	  {
		#print "line=$line\n";
		my $msg;
		($time, $sequenceNumber, $msg) = $line=~/^(\S+\s+\S+).*\s+ERRM\s+(\S+)\s+(.*)$/;
		#todo: parse the time out of the msg (real time the event occurred) and use instead of the time above from the audit log entry
		my ($type, $resource) = $msg=~/^(Event\s*:\s*|Rearm event\s*:\s*)(\S+)/i;
		#print "type=$type";
		if (!defined($type)) { next; }
		my $event = ($type=~/^Event/i ? 1 : 0);
		if ($event)   # set it to the time, in case it does not have a rearm expression and we want to display the time
		  {
			my ($mon, $d, $y, $h, $min, $s) = $time=~m|^(\d+)/(\d+)/(\d+)\s+(\d+):(\d+):(\d+)|;
			$event = main::timelocal($s, $min, $h, $d, $mon-1, $y);
		  }
		#print "$resource is $event\n";
		$$resStates{$resource} = $event;
	  }
	#foreach my $k (keys %$resStates) { print "<p>$k=$$resStates{$k}</p>"; }
	
	# Store the time and sequence # of the last event we got
	if (defined($time)) { $eventTime = GuiUtils->storeTimeAndSeqNum($time, $sequenceNumber, $LAST_EVENT_FILE); }
	#print "</p>\n";

	my $checkedTime = time();
	return ($resStates, $checkedTime, $eventTime);
  }
