#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display properties for a response

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;

# maps attribute names to cmd option to use to set it
my %attrOpt = ( ResponseName => '',
				Action => '-n',
				DaysOfWeek => '-d',
				TimeOfDay => '-t',
				ActionScript => '-s',
				ReturnCode => '-r',        # have to handle this and the next one specially
				CheckReturnCode => '',
				EventType => '-e',
				StandardOut => '-o',
				EnvironmentVars => '-E',
				UndefRes => '-u');

if (exists($::in{scriptButton}))
  {
	if (length($::in{inputActionScript})) { &redirect("../root$::in{inputActionScript}"); }
  }

my $create = exists($::in{resCreateButton}) || $::in{changeButton} eq 'Create';     # whether we are creating a new response or showing properties of an existing response
my $doit = exists($::in{changeButton});     # this means they pressed the Change or Create button from this page

GuiUtils->addHeader('res-props', 3, $create?'Create Response':'Response Properties');

my %values;
my ($response, $changedAttrStr);

if ($doit)           # creating a response or changing properties
  {
	if ($create)        # create response from the entries in the input fields
	  {
		my $cmd = 'mkresponse ' . buildAttrStr();
		$cmd .= ' ' . NodeUtils->quote($::in{inputResponseName});
		my $outref = GuiUtils->runcmd($cmd);
		$response = $::in{inputResponseName};
		if (!$::RUNCMD_RC) { $changedAttrStr = "Response $response created."; }
	  }
	else      # change properties
	  {
		# Get original values
		my %orig = split(/:\|:/, $::in{origValues});
		#print "<p>orig=", join(',',%orig), "</p>\n";

		# Figure out which attributes changed
		my %changedAttrs;
		foreach my $k (keys %::in)
		  {
			my $a;
			if (($a) = $k=~/^input(.+)$/) { if ($::in{$k} ne $orig{$a}) { $changedAttrs{$a} = NodeUtils->quote($::in{$k}); } }
		  }

		# If the name changed, we need to run a separate cmd
		$response = $orig{ResponseName};
		if (defined($changedAttrs{ResponseName}))
		  {
			my $outref = GuiUtils->runcmd(qq(chresponse -c $changedAttrs{ResponseName} $response));
			if (!$::RUNCMD_RC)
			  {
				$response = $changedAttrs{ResponseName};           # they changed it
				$changedAttrStr = 'Properties changed: ' . join(', ', keys %changedAttrs);
			  }
			delete $changedAttrs{ResponseName};
		  }

		# Now run the main chresponse cmd
		if (scalar(%changedAttrs))
		  {
			# We cannot change an action.  We have to delete it and add it back in.
			if (length($orig{Action}))
			  {
				my $outref = GuiUtils->runcmd(qq(chresponse -p $attrOpt{Action} ) . NodeUtils->quote($orig{Action}) . ' ' . NodeUtils->quote($response));
				#if (!$::RUNCMD_RC) {}
			  }
			
			my $attrStr = buildAttrStr();
			my $outref = GuiUtils->runcmd(qq(chresponse -a $attrStr ) . NodeUtils->quote($response));
			if (!$::RUNCMD_RC)
			  {
				if (length($changedAttrStr)) { $changedAttrStr .= ', ' . join(', ', keys %changedAttrs); }
				else { $changedAttrStr = 'Properties changed: ' . join(', ', keys %changedAttrs); }
			  }
		  }
		else { if (!length($changedAttrStr)) { $changedAttrStr = 'No properties were changed.'; } }
	  }
  }
else     # only displaying
  {
	$response = $::in{responseList};
  }

# Whether we are 1st coming to this page or have changed/created a response, display it
if (length($response))
  {
	my $cmd = qq(lsresponse -Ax -D ':|:' ) . NodeUtils->quote($response);
	my $outref = GuiUtils->runcmd($cmd);
	
	($values{ResponseName}, $values{Node}, $values{Action}, $values{DaysOfWeek}, $values{TimeOfDay}, $values{ActionScript}, $values{ReturnCode}, $values{CheckReturnCode}, $values{EventType}, $values{StandardOut}, $values{EnvironmentVars}, $values{UndefRes}) = split(/:\|:/, $$outref[0]);

	# lsreponse surrounds all the strings by double quotes
	foreach my $k (keys %values) { if ($values{$k}=~/^"/) { $values{$k} =~ s/^"(.*)"$/$1/; } else { $values{$k} =~ s/^'(.*)'$/$1/; } }
	#print "ResponseName=$values{ResponseName}\n";
  }

my $pageTitle = $create ? 'Create a Response' : "Properties for Response: $response";
my $pageDescription = $create ? 'Create a response by filling in property values and clicking on Create.' : 'View or change the properties of this response.';
my $changeButtonStr = $create ? 'Create' : 'Change';
my $nameDescription = $create ? 'Change this value when creating a new response.' : 'Changing this will change the name of the existing response (not create a new response).';
my $actionStr = $create ? '' : '  Modifying this name will add a new action to the response.  (For now, you must run "chresponse -p -n <i>action response</i>" to delete an action from a response.)';

my $code = GuiUtils->processHTML('resp-props-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('.', 'Monitor');


sub buildAttrStr
  {
	if (!length($::in{inputAction})) { return undef; }
	my $attrstr = "$attrOpt{Action} " . NodeUtils->quote($::in{inputAction});
	foreach my $k (keys %::in)
	  {
		my $a;
		if (($a) = $k=~/^input(.+)$/) # this is an attribute that can be inputted
		  {
			if (length($::in{$k})) # they actually specified something
			  {
				if ($a eq 'ResponseName') {} # we will add this at the end
				elsif ($a eq 'StandardOut' || $a eq 'UndefRes')
				  {
					if ($::in{$k}=~/^(y|1)$/i) { $attrstr .= " $attrOpt{$a}"; }
					# else do not add this flag
				  }
				elsif ($a eq 'CheckReturnCode')
				  {
					if ($::in{$k}=~/^(y|1)$/i && length($::in{inputReturnCode})) { $attrstr .= " $attrOpt{ReturnCode} " . NodeUtils->quote($::in{inputReturnCode}); }
				  }
				elsif ($a eq 'ReturnCode' || $a eq 'Action') {}	# we already handled these
				else { $attrstr .= " $attrOpt{$a} " . NodeUtils->quote($::in{$k}); }
			  }
		  }
	  }
	return $attrstr;
  }
