#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2003 by IBM Corp.
#
# Run the diagnose cmds on the node

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();

GuiUtils->addHeader('diagnode', 0, 'Run Diag Cmds', 1);

print "<H2>Diagnostic Commands and Output:</H2>\n";

my $dshcmd = "/opt/csm/bin/dsh -s -n $::in{nodes}";
$::GUI_VERBOSE = 1;      # always want to show them the cmds we are running on this page
if (exists($::in{pingButton}))     # ping the node
  {
	GuiUtils->runcmd("ping -c 10 $::in{nodes}", 1, {NoVerbose=>1});
	#GuiUtils->runcmd('echo "Probing.  This may take a while...";/opt/diagnostics/bin/probemgr', 1, {NoVerbose=>1});
	#GuiUtils->runcmd('/home/bp/bin/slowoutput', 1, {NoVerbose=>1});
	#for (my $i=1; $i<=5; $i++) { print "<p>$i</p>\n";  sleep 5; }
  }
elsif (exists($::in{testDshButton}))     # run dsh date
  {
	GuiUtils->runcmd("$dshcmd date", 1);
  }
elsif (exists($::in{runcmdButton}))     # run dsh
  {
	GuiUtils->runcmd("$dshcmd $::in{runcmdInput}", 1);
  }
elsif (exists($::in{viewFileButton}))     # view a file
  {
	GuiUtils->runcmd(qq($dshcmd 'if [ -d "$::in{viewFileInput}" ]; then { /bin/ls "$::in{viewFileInput}"; } else { cat "$::in{viewFileInput}"; } fi'), 1);
  }
elsif (exists($::in{csmLogsButton}))     # list /var/log/csm dir
  {
	GuiUtils->runcmd("$dshcmd /bin/ls /var/log/csm", 1);
  }
elsif (exists($::in{syncTimeButton}))     # set the node time to the mgmt svrs time
  {
	my ($sec,$min,$hour,$mday,$mon,$year) = gmtime();
	# the format for the date cmd is: %mnth%day%Hour%Min%Year.%Sec
	my $t = twoDigits($mon+1) . twoDigits($mday) . twoDigits($hour) . twoDigits($min) . ($year+1900) . '.' . twoDigits($sec);
	GuiUtils->runcmd("/opt/csm/bin/dsh -s -n $::in{nodes} 'date -u $t;date'", 1);
  }
elsif (exists($::in{probeButton}))     # run probemgr
  {
	GuiUtils->msg('I', 'Probing.  This may take a while...');
	#GuiUtils->runcmd('/opt/diagnostics/bin/probemgr', 1, {NoVerbose=>1});
	my $out = GuiUtils->runcmd("$dshcmd /opt/diagnostics/bin/probemgr -l 0", 3);
	GuiUtils->processProbeOutput($out, 1);
  }
elsif (exists($::in{lssrcButton}))     # run lssrc
  {
	GuiUtils->runcmd("$dshcmd lssrc -a", 1);
  }
elsif (exists($::in{stopsrcButton}))     # restart rmc
  {
	GuiUtils->runcmd("$dshcmd '/usr/sbin/rsct/bin/rmcctrl -z; /usr/sbin/rsct/bin/rmcctrl -s'", 1);
  }
elsif (exists($::in{csmagentrmTraceButton}))     # display the dmsrm trace file
  {
	GuiUtils->runcmd("$dshcmd '/usr/sbin/rsct/bin/rpttr /var/ct/IW/log/mc/IBM.CSMAgentRM/trace | tail -n 2000'", 1);
  }
elsif (exists($::in{errmTraceButton}))     # display the errm trace file
  {
	GuiUtils->runcmd("$dshcmd '/usr/sbin/rsct/bin/rpttr -o dtic /var/ct/IW/log/mc/IBM.ERRM/trace | tail -n 2000'", 1);
  }
elsif (exists($::in{hwctrlrmTraceButton}))     # display the hwctrlrm trace file - not implemented yet on the node
  {
	GuiUtils->runcmd('/usr/sbin/rsct/bin/rpttr /var/ct/IW/log/mc/IBM.HWCTRLRM/trace | tail -n 2000', 1, {NoVerbose=>1});
  }

print "<P><BR></P>\n";
&footer("index.cgi?nodes=$::in{nodes}", 'Diagnose Node');

sub twoDigits
  {
	my $v = $_[0];
	$v += 0;
	return ('0' x ($v<=9)) . $v;
  }
