#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display updatenode cmd results

use strict;
use lib $ENV{PWD};
require GuiUtils;
GuiUtils->init();
require InstallUtils;

#%access=&get_module_acl;

GuiUtils->addHeader('updatenode', 0, 'Update Nodes', 1);
print "<H2>Updatenode Command Output:</H2>\n";

if ($::in{nodepw}=~/\S/)
  {
	setupSSH($::in{nodepw}, split(/,/, $::in{nodes}));
	#$pwcmd = qq(/bin/echo -e "$::in{nodepw}\n" | $update_cmd);
	#if ($smscmd) { GuiUtils->runcmd($smscmd, 1); }
  }


if ($::in{Install} || $::in{CFM} || $::in{SMS})
  {
	my $update_cmd = "$GuiUtils::csmroot/bin/updatenode ";

	if (!($::in{Install} && $::in{CFM} && $::in{SMS}))    # if they are doing all 3, I think the updatenode cmd runs more efficiently if you do not specify these flags
	  {
		if($::in{Install}){ $update_cmd .= "-I "; }
		if($::in{CFM}){ $update_cmd .= "-F "; }
		if($::in{SMS}){ $update_cmd .= "-S "; }
	  }

	if($::in{Verbose}){ $update_cmd .= "-v "; }
	if($::in{Keys}){ $update_cmd .= "-k "; }
	if($::in{Test}){ $update_cmd .= "-t "; }
	if($::in{Force}){ $update_cmd .= "-f "; }
	if($::in{cfmoptions}){ $update_cmd .= '--cfmoptions ' . NodeUtils->quote($::in{cfmoptions}) . ' '; }
	if($::in{smsoptions}){ $update_cmd .= '--smsoptions ' . NodeUtils->quote($::in{smsoptions}) . ' '; }

	$update_cmd .= "--nopwd ";    #this keeps it from prompting for a password

	$update_cmd .= "-n $::in{nodes} ";

	# This was to overcome a bug in 1.3.0.20 in which sms does not work from updatenode if passed smsoptions
	#if($::in{SMS} && !$::in{smsoptions}){ $update_cmd .= "-S "; }
	#if($::in{smsoptions}){ $update_cmd .= '--smsoptions ' . NodeUtils->quote($::in{smsoptions}) . ' '; }
	#my $smscmd;
	#if($::in{smsoptions}){ $smscmd = "$GuiUtils::csmroot/bin/smsupdatenode -n $::in{nodes} $::in{smsoptions}"; }

	GuiUtils->runcmd($update_cmd, 1);
  }

print "<P><BR></P>\n";

&footer("update.cgi?nodes=$::in{nodes}", 'Update Nodes');


sub setupSSH
  {
	my $pw = shift @_;
	my @nodes = @_;
	$::REMOTESHELL_EXPECT = "$::CSMDIR/remoteshell.expect";
	$::REMOTE_SHELL = '/usr/bin/ssh';
	$::MKDIR = '/bin/mkdir';
	$::CAT = '/bin/cat';
	$::RM = '/bin/rm';
	
	my @badnodes = ();
	my $n_str = join ',', @nodes;
	$::SETUP_OTHER =1;
	InstallUtils->rmNodesKnownHosts(\@nodes);
	#system("$::REMOTESHELL_EXPECT -k"); #make the OpenSSH keys
	GuiUtils->runcmd("$::REMOTESHELL_EXPECT -k"); #make the OpenSSH keys
	InstallUtils->cpSSHFiles();
	my $SSHdir = "/csminstall/csm/config/.ssh";
	open (FILE, ">$SSHdir/copy.perl") or die "cannot open file $SSHdir/copy.perl\n";
	
	print FILE "#!/usr/bin/perl
my \@root = split ':', (`/bin/grep ^root /etc/passwd 2>&1`);
my \$home = \$root[5];
umask(0077);
\$dest_dir = \"\$home/.ssh/\";
if (! -d \"\$dest_dir\" ) {
    # create a local directory
    \$cmd = \"$::MKDIR -p \$dest_dir\";
    system(\"\$cmd\"); 
    chmod 0700, \$dest_dir;
}
`$::CAT /tmp/.ssh/authorized_keys >> \$home/.ssh/authorized_keys 2>&1`;
`$::CAT /tmp/.ssh/authorized_keys2 >> \$home/.ssh/authorized_keys2 2>&1`;
`$::RM -f /tmp/.ssh/authorized_keys 2>&1`;
`$::RM -f /tmp/.ssh/authorized_keys2 2>&1`;
`$::RM -f /tmp/.ssh/copy.perl 2>&1`;
rmdir(\"/tmp/.ssh\");";
	close FILE;
	chmod 0744, "$SSHdir/copy.perl";
	#set an ENV var if more than 10 nodes
	my $num_nodes = scalar(@nodes);
	if($num_nodes > 10){
		$ENV{'CSM_UPD_MULTNODES'}=1;
	}
	$ENV{CSM_REMOTE_PASSWORD} = $pw;
	#system("$::REMOTESHELL_EXPECT -s $n_str");
	GuiUtils->runcmd("$::REMOTESHELL_EXPECT -s $n_str");
	# must always check to see if worked
	my $check_cmd = "$::REMOTESHELL_EXPECT -t $::REMOTE_SHELL";
	foreach my $n (@nodes){
			#`$check_cmd $n 2>&1`;
			GuiUtils->runcmd("$check_cmd $n", 2);
			#my $check_rc = $? >> 8;
			#if($check_rc){
			if($::RUNCMD_RC){
				push @badnodes, $n;
			}
	}
	#system("stty echo");
	delete $ENV{'CSM_UPD_MULTNODES'};
	my ($MSGCAT, $MSGMAPPATH, $MSGSET) = ('csmInstall.cat', "$GuiUtils::csmroot/msgmaps", 'csminstall');
	if(@badnodes){
		my $nstring = join ',', @badnodes;
		#NodeUtils->message( 'E', 'EMsgRemoteShellSetupFailed', $nstring);
		GuiUtils->msg('E', NodeUtils->getMessageFromCat($MSGCAT, $MSGMAPPATH, $MSGSET, 'EMsgRemoteShellSetupFailed', 'exec_updatenode.cgi', $nstring));
		return @badnodes;
	}
	else{
		#NodeUtils->message('I', 'IMsgRemoteSetupComplete', $::REMOTE_SHELL);
		GuiUtils->msg('I', NodeUtils->getMessageFromCat($MSGCAT, $MSGMAPPATH, $MSGSET, 'IMsgRemoteSetupComplete', $::REMOTE_SHELL));
		return 0;
	}
}
