#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Do HW Ctrl on the nodes

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();


my %values = ();
my %orig = ();


GuiUtils->splitInputValues(\%values, \%orig);

my $nodes = $values{nodes} if (exists($values{nodes}));
my @node_array = split(/,/, $nodes);


GuiUtils->addHeader('hwctrl', 0, 'HW Ctrl');


#foreach my $env (keys %ENV) {
#	print(qq(<p> $env : $ENV{$env} </p>));
#}



if ($nodes ne '' && $orig{Initialized} == 1) {
	$values{Initialized} = $orig{Initialized};
	$values{InstallDistributionName} = $orig{InstallDistributionName};
	$values{InstallDistributionVersion} = $orig{InstallDistributionVersion};
	$values{InstallMethod} = $orig{InstallMethod};
} else {
	initValues();
}

# Connetions
my $connectionStatusImg = "yellow-ball-m.gif";
my $powerStatusImg = "yellow-ball-m.gif";

if (exists($values{testButton})) { 
	my ($rc, $hwInfo) = GuiUtils->getHcpHwInfo(\%values, \@node_array);
	setConnectionStatus($rc);
	delete ($values{testButton});
} else {
	if (exists($orig{PS})) {
		setConnectionStatus($orig{PS});
	} else {
		my ($rc, $hwInfo) = GuiUtils->getHcpHwInfo(\%values, \@node_array);
		setConnectionStatus($rc);
	}
}


if (exists($values{resethcpButton})) { 
	GuiUtils->runcmd("$GuiUtils::csmroot/bin/rpower -n $nodes resetsp_hcp");
	delete ($values{resethcpButton});
	my ($rc, $hwInfo) = GuiUtils->getHcpHwInfo(\%values, \@node_array);
	setConnectionStatus($rc);
} 

if (exists($values{queryStatusButton})) {
	powerStatus();
	delete ($values{queryStatusButton});
} elsif ($values{InitPowerStatus} == 1) { 
	powerStatus();
	delete ($values{InitPowerStatus});
} else {
	if (exists($orig{PowerOnFlag}) || exists($orig{PowerOffFlag}) || exists($orig{PowerUnknownFlag})) {
		if ($orig{PowerOnFlag} == 1) {
			$values{PowerOn} = $orig{PowerOn};
			$values{PowerOnFlag} = $orig{PowerOnFlag};
		}
		if ($orig{PowerOffFlag} == 1) {
			$values{PowerOff} = $orig{PowerOff};
			$values{PowerOffFlag} = $orig{PowerOffFlag};
		}
		if ($orig{PowerUnknownFlag} == 1) {
			$values{PowerUnknown} = $orig{PowerUnknown};
			$values{PowerUnknownFlag} = $orig{PowerUnknownFlag};
		}
	} else {
		initPowerStatus();
	}
	
}

if (exists($values{rpowerButton})) { 
	my $action;
	if ($values{rpowerRadio} eq 'powerOn') {
		$action = "on";
	} elsif($values{rpowerRadio} eq 'powerOff') {
		$action = "off";
	} elsif($values{rpowerRadio} eq 'reboot') {
		$action = "reboot";
	} elsif($values{rpowerRadio} eq 'resetNodeSP') {
		$action = "resetsp_host";
	} elsif($values{rpowerRadio} eq 'resetHCP') {
		$action = "resetsp_hcp";
	} elsif($values{rpowerRadio} eq 'powerOffSoft') {
		$action = "/sbin/poweroff";
	} elsif($values{rpowerRadio} eq 'rebootSoft') {
		$action = "/sbin/reboot";
	}

	delete ($values{rpowerButton});
	delete ($values{rpowerRadio});

	if ($action =~ /^\/sbin\//)          # dsh to the nodes
	  {
		GuiUtils->runcmd("$GuiUtils::csmroot/bin/dsh -s -n $nodes $action"); 
	  }
	else          # rpower cmd
	  {
		GuiUtils->runcmd("$GuiUtils::csmroot/bin/rpower -n $nodes $action"); 
		powerStatus();
	  }
}

#print("<p> InstallDistributionName: $values{InstallDistributionName}</p>");
#print("<p> InstallDistributionVersion: $values{InstallDistributionVersion}</p>");
#print("<p> InstallMethod: $values{InstallMethod}</p>");

if (exists($values{getmacButton})) { 
	my %adapterGroup = ();
	my $idn = GuiUtils->parseValues("InstallDistributionName", \%values, \@node_array);
	my $idv = GuiUtils->parseValues("InstallDistributionVersion", \%values, \@node_array);
	my $im = GuiUtils->parseValues("InstallMethod", \%values, \@node_array);
	foreach my $node (@node_array) {
		my $ag=$$idn{$node}.$$idv{$node}.$$im{$node};
		if(exists($adapterGroup{$ag})) {
			$adapterGroup{$ag} .= ",$node";
		} else {
			$adapterGroup{$ag} = $node;
		}
	}

	foreach my $group (keys %adapterGroup) {
		GuiUtils->runcmd("$GuiUtils::csmroot/bin/getadapters -n $adapterGroup{$group} -w"); 
		#print("<p>$group, $adapterGroup{$group}</p>");
	}
	initValues("InstallAdapterMacaddr");
	delete ($values{getmacButton});
}

if (exists($values{mtmsButton})) { 
	my %mtms = ();
	my $snm = GuiUtils->parseValues("HWSerialNum", \%values, \@node_array);
	my $type = GuiUtils->parseValues("HWType", \%values, \@node_array);
	my $model = GuiUtils->parseValues("HWModel", \%values, \@node_array);

	foreach my $node (@node_array) {
		$mtms{HWSerialNum}{$node} = $$snm{$node};
		$mtms{HWType}{$node} = $$type{$node};
		$mtms{HWModel}{$node} = $$model{$node};
	}

	my $cmd = "$GuiUtils::csmroot/bin/lshwstat -n $nodes serialnum";
	my $serialNums = GuiUtils->runcmd("LANG=C $cmd", 3);
	
	while (my $line=<$serialNums>) {
		if ($line =~ m/^\S+:\s+Serial\s+Number:\s+\S+\s+$/) {
			my ($host, $snName, $sn) = split(/:\s+/, $line);
			foreach my $node (@node_array) {
				if ($host eq $node) {
					$mtms{HWSerialNum}{$host} = $sn;
					last;
				}
			}
		} elsif ($::RUNCMD_RC) {
			GuiUtils->msg('E', $line);
		} else {
			GuiUtils->msg('V', $line);
		}
	}
	close($serialNums);
	

	$cmd = "$GuiUtils::csmroot/bin/lshwstat -n $nodes model";
	my $models = GuiUtils->runcmd("LANG=C $cmd", 3);

	while (my $line=<$models>) {
		if ($line =~ m/^\S+:\s+Machine\s+Model:\s+\S+\s+$/) {
			my ($host, $mm, $mode) = split(/:\s+/, $line);
			foreach my $node (@node_array) {
				if ($host eq $node) {
					$mtms{HWType}{$host} = 
						substr($mode, 0, 4);
					$mtms{HWModel}{$host} = 
						substr($mode, 4);
					last;
				}
			}
		} elsif ($::RUNCMD_RC) {
			GuiUtils->msg('E', $line);
		} else {
			GuiUtils->msg('V', $line);
		}
	}
	close($models);

	# Save the changed values
	my $hwDBInfo = getValuesFromCSMDB();
	foreach my $node (@node_array) {
		my $cmd = "$GuiUtils::csmroot/bin/chnode -n $node";
		my $cmdTmp = $cmd;
		if (!GuiUtils->isEmpty($mtms{HWSerialNum}{$node})) {
			$cmd .= " HWSerialNum=\"$mtms{HWSerialNum}{$node}\"";
		}
		if (!GuiUtils->isEmpty($mtms{HWType}{$node})) {
			$cmd .= " HWType=\"$mtms{HWType}{$node}\"";
		}
		if (!GuiUtils->isEmpty($mtms{HWModel}{$node})) {
			$cmd .= " HWModel=\"$mtms{HWModel}{$node}\"";
		}
		if ($cmd ne $cmdTmp) {
			GuiUtils->runcmd("$cmd");
		}
	}
	GuiUtils->setValues("HWSerialNum", \%values, \%mtms);
	GuiUtils->setValues("HWType", \%values, \%mtms);
	GuiUtils->setValues("HWModel", \%values, \%mtms);

	delete ($values{mtmsButton});
}

if (exists($values{saveAttrsButton})) { 
	my $hwDBInfo = getValuesFromCSMDB();

	my $valueHWControlPoint = 
	GuiUtils->parseValues("HWControlPoint", \%values, \@node_array);

	my $valuePowerMethod = 
	GuiUtils->parseValues("PowerMethod", \%values, \@node_array);

	my $valueHWControlNodeId = 
	GuiUtils->parseValues("HWControlNodeId", \%values, \@node_array);

	my $valueInstallAdapterMacaddr = 
	GuiUtils->parseValues("InstallAdapterMacaddr", \%values, \@node_array);

	my $valueHWType = 
	GuiUtils->parseValues("HWType", \%values, \@node_array);

	my $valueHWModel = 
	GuiUtils->parseValues("HWModel", \%values, \@node_array);

	my $valueHWSerialNum = 
	GuiUtils->parseValues("HWSerialNum", \%values, \@node_array);

	foreach my $node (@node_array) {
		my $cmd = "$GuiUtils::csmroot/bin/chnode -n $node ";
		my $cmdTmp = $cmd;

		if ($$valueHWControlPoint{$node} ne 
			$$hwDBInfo{HWControlPoint}{$node}) {
			$cmd .= " HWControlPoint=\"$$valueHWControlPoint{$node}\"";
		}

		if ($$valuePowerMethod{$node} ne 
			$$hwDBInfo{PowerMethod}{$node}) {
			$cmd .= " PowerMethod=\"$$valuePowerMethod{$node}\"";
		}

		if ($$valueHWControlNodeId{$node} ne 
			$$hwDBInfo{HWControlNodeId}{$node}) {
			$cmd .= " HWControlNodeId=\"$$valueHWControlNodeId{$node}\"";
		}

		if ($$valueInstallAdapterMacaddr{$node} ne 
			$$hwDBInfo{InstallAdapterMacaddr}{$node}) {
			$cmd .= " InstallAdapterMacaddr=\"$$valueInstallAdapterMacaddr{$node}\"";
		}

		if ($$valueHWType{$node} ne 
			$$hwDBInfo{HWType}{$node}) {
			$cmd .= " HWType=\"$$valueHWType{$node}\"";
		}

		if ($$valueHWModel{$node} ne 
			$$hwDBInfo{HWModel}{$node}) {
			$cmd .= " HWModel=\"$$valueHWModel{$node}\"";
		}

		if ($$valueHWSerialNum{$node} ne 
			$$hwDBInfo{HWSerialNum}{$node}) {
			$cmd .= " HWSerialNum=\"$$valueHWSerialNum{$node}\"";
		}

		if ($cmd ne $cmdTmp) {
			GuiUtils->runcmd("$cmd");
		}
	}

	delete ($values{saveAttrsButton});
}


my $code = GuiUtils->processHTML('index-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('', 'CSM Home');

sub initPowerStatus {
	$values{PowerOnFlag} = 0;
	$values{PowerOffFlag} = 0;
	$values{PowerUnknownFlag} = 0;
	$values{PowerOn} ="";
	$values{PowerOff} ="";
	$values{PowerUnknown} ="";

	my $ps = GuiUtils->parseValues("PowerStatus", \%values, \@node_array);
	foreach my $node (@node_array) {
		#print(qq(<p>PowerStatus: $node=$$ps{$node}</p>));
		if ($$ps{$node} == 0) {
			$values{PowerOffFlag} = 1;
			$values{PowerOff} .= ", $node";
		} elsif ($$ps{$node} == 1) {
			$values{PowerOnFlag} = 1;
			$values{PowerOn} .= ", $node";
		} else {
			$values{PowerUnknownFlag} = 1;
			$values{PowerUnknown} .= ", $node";
		}
	}
	$values{PowerOn} =~ s/^, //;
	$values{PowerOff} =~ s/^, //;
	$values{PowerUnknown} =~ s/^, //;
}

sub powerStatus {
	$values{PowerOnFlag} = 0;
	$values{PowerOffFlag} = 0;
	$values{PowerUnknownFlag} = 0;
	$values{PowerOn} ="";
	$values{PowerOff} ="";
	$values{PowerUnknown} ="";

	# Get the node list
	my @nodeList = @node_array;

	# Run rpower command and return the command resoults in a filehandle.
	my $pStatus = GuiUtils->runcmd("LANG=C $GuiUtils::csmroot/bin/rpower -n $nodes query", 3); 
	while (my $line = <$pStatus>) {
		my ($host, $status) = split (/\s+/, $line);
		if (($line =~ m/^\S+\s+\S+\s+$/) && 
		(($status eq "on") || ($status eq "off"))) {
			for my $i (0 .. $#nodeList) {
				if ($nodeList[$i] eq $host) {
					if ($status eq "on") {
						# Put node in the on list
						$values{PowerOnFlag} = 1;
						$values{PowerOn} .= 
							", $nodeList[$i]";
					} elsif ($status eq "off") {
						# Put node in the off list
						$values{PowerOffFlag} = 1;
						$values{PowerOff} .= 
							", $nodeList[$i]";
					}
					delete($nodeList[$i]);
					#print("<p>$host, $status</p>");
				} 
			}
		} elsif ($::RUNCMD_RC) {
			GuiUtils->msg('E', $line);
		} else {
			GuiUtils->msg('V', $line);
		}
		#print("<p>$line</p>");
	}
	close($pStatus);

	# Put the rest of nodes to the unknown list.
	for my $i (0 .. $#nodeList) {
		if (exists($nodeList[$i])) {
			$values{PowerUnknownFlag} = 1;
			$values{PowerUnknown} .= ", $nodeList[$i]";
		}
	}

	$values{PowerOn} =~ s/^, //;
	$values{PowerOff} =~ s/^, //;
	$values{PowerUnknown} =~ s/^, //;
}

sub setConnectionStatus {
	my $status = shift;
	if ($status == 0) {
		$connectionStatusImg = "green-ball-m.gif";
	} elsif ($status == 1) {
		$connectionStatusImg = "blue-ball-m.gif";
	} else {
		$connectionStatusImg = "red-ball-m.gif";
	}
	$values{PS} = $status;
}

sub initValues {
	my $attribute = shift;

	my $hohHwInfo = getValuesFromCSMDB();
	foreach my $a (keys %$hohHwInfo) {
		if (!exists($values{$a}) || $a eq $attribute) {
			GuiUtils->setValues($a, \%values, $hohHwInfo);
		}
	}
	$values{Initialized} = 1;
}

sub getValuesFromCSMDB {
	# Get hw attribute from CSM MS.
	my $where = NodeUtils->buildWhereInList(\@node_array, 'Hostname');
	my $what = "Hostname::PowerMethod::HWControlPoint::HWControlNodeId::LParID::HWType::HWModel::HWSerialNum::InstallAdapterMacaddr::PowerStatus::Status::InstallDistributionName::InstallDistributionVersion::InstallMethod";
	my $exist_attr = GuiUtils->runcmd("LANG=C /usr/bin/lsrsrc-api -D ':|:' -n -i -s IBM.ManagedNode::${where}::${what}", 3);

	my %hohHwInfo = ();

	while (my $line = <$exist_attr>) {
		if ($line =~ m/^Hostname:|:\s+:|:PowerMethod:|:\s+:|:\S+/) {
			my %attrs = split(/:\|:/, $line);
			my $hostname = $attrs{Hostname};
			delete $attrs{Hostname};
			foreach my $a (keys %attrs) {
				$hohHwInfo{$a}{$hostname} = $attrs{$a};
				delete $attrs{$a};
			}
		} elsif ($::RUNCMD_RC) {
			GuiUtils->msg('E', $line);
		} else {
			GuiUtils->msg('V', $line);
		}
	}
	close($exist_attr);

	return(\%hohHwInfo);
}

