#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display main page of CSM Web Interface

use strict;
use lib '$ENV{PWD}';
require GuiUtils;
GuiUtils->init();

# Build node list
#my $nodeErr;
my ($options, @nodes);

# Go to the page corresponding to the button
if ($::in{propButton}) { &redirect("properties.cgi?" . getParams()); }
elsif ($::in{updateButton}) { &redirect("update.cgi?" . getParams()); }
elsif ($::in{runcmdButton})
  {
	my $p = getParams();
	if ($p =~ /nodes=./) { &redirect("dsh.cgi?$p"); }          # some nodes were selected
	else { &redirect('../shell'); }                                  # no nodes selected
  }
elsif ($::in{copyFilesButton}) { &redirect("dcp.cgi?" . getParams()); }
elsif ($::in{createGroupButton}) { &redirect("nodegrp.cgi?" . getParams() . "&groupType=staticType"); }
elsif ($::in{hwctrlButton}) { &redirect("hwctrl/index.cgi?" . getParams()); }
elsif ($::in{rsaButton})
  {
	my $node = getFirstNode();
	my $nodeAttrs = NodeUtils->listNodeAttrs([$node], 'HWControlPoint');
	my $hcp = $$nodeAttrs[0];
	#&main::PrintHeader();
	#print "<p>hcp=$hcp</p>\n";
	if (length($hcp)) { &redirect("http://$hcp"); }
	else { &redirect('jserror.cgi?error=' . &urlize("The HWControlPoint attribute is not defined for $node.")); }
  }
elsif ($::in{perfmonButton}) { &redirect("perfmon/index.cgi?" . getParams()); }
elsif ($::in{webminButton}) { &redirect('http://' . getFirstNode() . ':10000'); }
elsif ($::in{diagButton}) { &redirect("diagnode/index.cgi?" . getParams()); }
elsif ($::in{installButton}) { &redirect("install/index.cgi?" . getParams()); }
elsif ($::in{removeButton}) { &redirect("remove.cgi?" . getParams()); }
# else they are coming to this page from elsewhere, so display it...


$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
$ENV{'CT_MANAGEMENT_SCOPE'}=1;
if (length($::config{CSM_NODE_PREFIX})) { $ENV{CSM_NODE_PREFIX} = $::config{CSM_NODE_PREFIX}; }

#%access=&get_module_acl;
## put in ACL checks here if needed

# Create title and header area
GuiUtils->addHeader('intro', 0);
#&header('Cluster Systems Management', 'images/csmlogo.gif', 'intro', 1, 1, undef,
#        '<A HREF=mailto:cluster\@us.ibm.com?Subject=CSM Web Interface Suggestion>Send suggestions to the authors</A>');

#print "config_directory=$::config_directory\n";
#if ($nodeErr) { GuiUtils->msg('E', $nodeErr); }

# Get list of nodes and attributes and store for the HTML
#my $lsnode_option;
#if ($::in{group}) { $lsnode_option = "-N $::in{group} "; }
my $noderef;
my $opthashref;
if ($::in{nodeRange} =~ /\S/) { $noderef = NodeUtils->gatherNodeList([$::in{nodeRange}]); }
elsif ($::in{group})
  {
	$noderef = NodeUtils->gatherNodeList(undef, { NodeGrp => $::in{group} });
	$opthashref = { SkipResolve => 1, Res2Unres => {} };   # node groups are already resolved and unduped
  }
#my @nodes;
#if ($::in{nodeRange} =~ /\S/) { push @nodes, $::in{nodeRange}; }
#elsif ($::in{group})
#  {
#	my $outref = NodeUtils->listNodeGroupExpMem($::in{group});
#	my ($name, $nodelist) = $$outref[0] =~ /^(.+):\|:\{(.*)\}$/;
#	@nodes = split(/,/, $nodelist);
#  }
#chop(@nodes = `/opt/csm/bin/lsnode $lsnode_option 2>&1`);
#if( ($? >>8) != 0){ @nodeAttrs = (); }  #if lsnodes failes
my $attrs = 'Hostname,HWType,InstallDistributionName,InstallDistributionVersion,Mode,Status';
my $nodeAttrs = NodeUtils->listNodeAttrs($noderef, $attrs, $opthashref);
@$nodeAttrs = sort @$nodeAttrs;
#foreach my $n (@$nodeAttrs) { print "<P>$n</P>\n"; }

# Get list of node group names for the select menu
my @nodegrp;
#chop(@nodegrp = `/opt/csm/bin/nodegrp`);
chop(@nodegrp = `lsrsrc-api -i -D ':|:' -s IBM.NodeGroup::::Name`);
@nodegrp = sort @nodegrp;

# Get versions of csm.server and xcsm.web
my %rpmVersions = GuiUtils->getInstalledVersions(1, qw(csm.server xcsm.web));

# Process the perl code within the HTML
#print '<P>Lsnode output: ' . join(',',@out) . '</P>';
my $code = GuiUtils->processHTML('index-src.html');
#open WEBCODE, '>/tmp/index-src.html'; select WEBCODE;
eval $code;
if (length($@)) { print "<P>$@</P>"; }

#if ($nodeErr) { print qq(<script language="JavaScript">if (document.getElementById('displayederror')) {alert('yes');} else {alert('no');}</script><div ID=displayederror></div>); }

&footer("/", 'Webmin Cluster Index');


sub getParams
  {
	my @nodes;
	my @params;
	foreach my $k (keys %::in)
	  {
		if ($k =~ /^node\d+/) { push @nodes, $::in{$k}; }
		elsif ($k ne 'nodegrps' && $k ne 'nodeRange') { push @params, "$k=" . &urlize($::in{$k}); }
	  }
	#push @params, 'nodes=' . &urlize(join(',', @nodes));  # is it necessary to urlize this?
	push @params, 'nodes=' . join(',', @nodes);
	return join('&', @params);
  }

sub getFirstNode
  {
	foreach my $k (keys %::in) { if ($k =~ /^node\d+/) { return $::in{$k}; } }
	return;
  }
