#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display performance monitoring info for the selected nodes

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;
## put in ACL checks here if needed

# Go to the page corresponding to the button
if ($::in{refresh})      # act as the hidden refresh frame for the install monitor page
  {
	GuiUtils->addRefreshHeader();
	my $nodeAttrs = installmon(1, $::in{nodes});
	my $resStates = {};
	foreach my $na (@$nodeAttrs)
	  {
		my ($node, $mode, $percent, $file) = split(/:\|:/, $na);
		my ($shortname) = split(/\./, $node);   # the dots mess up the name of the html object
		$$resStates{"$shortname"} = "${percent}::$file";
	  }
	my $checkedTime = time();
	GuiUtils->addRefreshBody('installmon', $resStates, $checkedTime, $checkedTime);
	return;
  }
elsif ($::in{consoleButton}) { &redirect("console.cgi?" . getParams()); }

GuiUtils->addHeader('installmon', 0, 'Install Monitoring');

#GuiUtils->initEvents('.LastInstallEventTime');

my $nodeAttrs = installmon(0, $::in{nodes});

my $code = GuiUtils->processHTML('monitor-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

#&footer('.', 'Install Nodes');

#&footer("./index.cgi?nodes=$::in{nodes}", 'Install Nodes');
my %rnt = ();
$rnt{nodes} = $::in{nodes};
GuiUtils->addFooter('./index.cgi', 'Install Nodes', \%rnt);


# Get the current installation progress of each node passed in on the URL
sub installmon
  {
	my ($refresh, $nodestr) = @_;

	my @nodes = split(/,/, $nodestr);
	#my $where = qq/Mode='Installing' && Hostname IN ('/ . join("','", @nodes) . q/')/;
	my $attrs = 'Hostname,Mode';
	if (!$refresh) { $attrs .= ',InstallDistributionName,InstallDistributionVersion'; }
	$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
	if ($::VERBOSE) { print '<P>'; }    # so the printing from listNodeAttrs is in a paragraph
	#my $nodeAttrs = NodeUtils->listNodeAttrs([], $attrs, { WhereStr=>$where });
	my $nodeAttrs = NodeUtils->listNodeAttrs(\@nodes, $attrs);
	
	foreach my $na (@$nodeAttrs)
	  {
		my ($node, $mode) = split(/:\|:/, $na);
		my ($percent, $file);
		if ($mode eq 'Installing')
		  {
			my $out = GuiUtils->runcmd(qq(ping -w 1 -c 1 $node), 2, {NoVerbose=>1});
			if (!$::RUNCMD_RC)
			  {
				$out = GuiUtils->runcmd(qq(/opt/csm/csmbin/nodestat.awk $node), 2, {NoVerbose=>1});
				if (!$::RUNCMD_RC && scalar(@$out))
				  {
					($percent, $file) = $$out[0] =~ /^\s*(\S+)\s*%\s*:\s*(.*)$/;
					if (!defined($file)) { $percent=0;  $file=$$out[0]; }
				  }
				else { ($percent, $file) = getStatus($node);  if (!defined($file)) { $file = 'preparing...'; } }
			  }
			else { ($percent, $file) = getStatus($node);  if (!defined($file)) { $file = 'pinging...'; } }
		  }
		elsif ($mode eq 'PreManaged') { $percent = 0;  $file = $mode; }
		elsif ($mode eq 'Managed' || $mode eq 'MinManaged') { $percent = 100;  $file = $mode; }
		$na .= ":|:$percent:|:$file";
	  }
	#foreach my $k (keys %$resStates) { print "<p>$k=$$resStates{$k}</p>"; }

	return $nodeAttrs;
  }


# This is called when we cannot ping the node or run nodestat.awk.  We have to decided if
# it is 0% installed or 100% installed.  Returns the percent and msg.
sub getStatus
  {
	my $node = $_[0];
	my $percent = 0;    my $msg = undef;
	my $out = GuiUtils->runcmd(qq(tail -1 /csminstall/csm/status/$node), 2, {NoVerbose=>1});
	if (!$::RUNCMD_RC && scalar(@$out))
	  {
		($msg) = $$out[0] =~ /^.*:\s(.*?)\.?\s*$/;
		#$msg =~ s/\.\s*$//;      # remove trailing period and spaces, if any
		if (defined($msg) && !grep(/$msg/, ('Full Install Initiated','Rebooting and Installing Node'))) { $percent = 100; }
	  }
	return ($percent, $msg);
  }

sub getParams
  {
	my @nodes;
	my @params;
	foreach my $k (keys %::in)
	  {
		if ($k =~ /^node\d+/) { push @nodes, &urlize($::in{$k}); }
		elsif ($k eq 'historyTimeTextfield') { push @params, "$k=" . &urlize($::in{$k}); }
		#else { push @params, "$k=" . &urlize($::in{$k}); }   # this gets too many
	  }
	#push @params, 'nodes=' . &urlize(join(',', @nodes));  # is it necessary to urlize this?
	if (scalar(@nodes)) { push @params, 'nodes=' . join(',', @nodes); }
	return join('&', @params);
  }
