#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display performance history for selected nodes

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();

GuiUtils->addHeader('perfdetails', 0, 'Detailed Metrics', 1);

my $node = $::in{nodes};   # for now assume on 1 node for this page

my $historyTime = $::in{historyTimeTextfield} || 10;

print "<H2>Detailed Performance Metrics for: $node</H2>\n";

#$::GUI_VERBOSE = 1;      # always want to show them the cmds we are running on this page

# Check if gnuplot is installed, since it is needed to show the graphs
my %rpmVersions = GuiUtils->getInstalledVersions(0, qw(gnuplot autoupdate));
if (!exists($rpmVersions{gnuplot}))       # try to download gnuplot
  {
	if (exists($rpmVersions{autoupdate}))
	  {
		GuiUtils->msg('I', 'Gnuplot is required to show the graphs, but currently not installed.  Trying to download it...');
		GuiUtils->runcmd('/usr/sbin/autoins --updatedir /tmp/csm/RedHat/9 --url http://ftp.redhat.com/pub/redhat/linux/9/en/os/i386/RedHat/RPMS gnuplot', 1, {NoVerbose=>1});
	  }
	else
	  {
		GuiUtils->msg('E', 'Gnuplot is required to show the graphs, but currently not installed.');
		doFooter();
		return;
	  }
  }

# Get a name for the data file for each condition we will plot
my $cpufile = GuiUtils->tmpname('dat');
my $realmemfile = GuiUtils->tmpname('dat');
my $pagingfile = GuiUtils->tmpname('dat');

# Write the data to the data files
GuiUtils->msg('V', "Writing CPU data to $cpufile, real memory data to $realmemfile, and paging space data to $pagingfile");
my $cmd = "./perfmondata -t $historyTime -n $node AnyNodeProcessorsIdleTimeChanged $cpufile AnyNodePctRealMemFreeChanged $realmemfile AnyNodePctTotalPgSpFreeChanged $pagingfile";
GuiUtils->runcmd($cmd);

my ($foundcpu, $foundrealmem, $foundpaging) = (0, 0, 0);

# Produce the png files and create html to display them
print qq(<TABLE> <COL span="1" align="center"> <TBODY>\n);

print qq(<TR> <TD align="center"><H2>CPU Time Used on Node $node</H2></TD> </TR>\n);
if (-s $cpufile)
  {
	my $cpupng = GuiUtils->tmpname('png');
	plotdata($cpufile, $cpupng);
	print qq(<TR> <TD><IMG src="$::CSMDIR/root$cpupng" border="0"></TD> </TR>\n);
  }
else { print qq(<TR> <TD align="center">No data available for the given time range.<BR><BR></TD> </TR>\n); }

print qq(<TR> <TD align="center"><H2>Real Memory Used on Node $node</H2></TD> </TR>\n);
if (-s $realmemfile)
  {
	my $realpng = GuiUtils->tmpname('png');
	plotdata($realmemfile, $realpng);
	print qq(<TR> <TD><IMG src="$::CSMDIR/root$realpng" border="0"></TD> </TR>\n);
  }
else { print qq(<TR> <TD align="center">No data available for the given time range.<BR><BR></TD> </TR>\n); }

print qq(<TR> <TD align="center"><H2>Paging Space Used on Node $node</H2></TD> </TR>\n);
if (-s $pagingfile)
  {
	my $pagingpng = GuiUtils->tmpname('png');
	plotdata($pagingfile, $pagingpng);
	print qq(<TR> <TD><IMG src="$::CSMDIR/root$pagingpng" border="0"></TD> </TR>\n);
  }
else { print qq(<TR> <TD align="center">No data available for the given time range.<BR><BR></TD> </TR>\n); }
print qq(</TBODY> </TABLE> <BR>\n);

  
#&footer('.', 'Performance Monitor');
doFooter();


sub doFooter { &footer('.', 'Performance Monitor'); }


sub plotdata
  {
	my ($datafile, $pngfile) = @_;
	GuiUtils->msg('V', "Creating graph for $datafile in $pngfile");

	# Run gnuplot and pipe in the control file
	my $cmd = '/usr/bin/gnuplot';
	if (!open(GNUPLOT, "| $cmd")) { GuiUtils->msg('E', "Can not open/run $cmd.  Error message:  $!"); return; }
	#if (!open(GNUPLOT, "> /tmp/gnuplot.in")) { GuiUtils->msg('E', "Can not open/run $cmd.  Error message:  $!"); return 0; }
	print GNUPLOT <<"END_OF_INPUT";
#set title "CPU % Used from ? to ?"
#set xlabel "Time"
set ylabel "CPU % Used"
set autoscale
set terminal png color
set out "$pngfile"
set bmargin 7
set grid
set timefmt "%H:%M:%S"
set xdata time
set format x "%H:%M:%S"
set xtics rotate
#plot  "cpu.dat" using 1:2 notitle with linespoints lt 3 pt 6
plot  "$datafile" using 1:2 notitle with linespoints lt 9 pt 9
END_OF_INPUT
	if (!close(GNUPLOT)) { GuiUtils->msg('E', "Exit code " . ($?>>8) . " from command $cmd.  Error message from cmd: $!."); return 0; }
	#GuiUtils->runcmd("$cmd < /tmp/gnuplot.in");

	# Remove data file
	unlink($datafile);
	#todo: when do we get rid of the pngfile?
	return 1;
  }
