#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display performance monitoring info for the selected nodes

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();
#%access=&get_module_acl;
## put in ACL checks here if needed
my $LAST_EVENT_FILE = '.LastPerfEventTime';     # the place to save the last sequence # we have read in the auditlog

# Go to the page corresponding to the button
if ($::in{refresh})      # act as the hidden refresh frame for the perfmon page
  {
	GuiUtils->addRefreshHeader();
	my ($resStates, $checkedTime, $eventTime) = perfmon($::in{nodes});
	GuiUtils->addRefreshBody('perfmon', $resStates, $checkedTime, $eventTime);
	return;
  }
elsif ($::in{detailsButton}) { &redirect("details.cgi?" . getParams()); }


# These are the definitions of the conditions that will be created, if necessary.  This is a hash in
# which the key is the name of the condition and the value is all the attribute
# name & value pairs separated by double colons (in the mkrsrc-api format).
my %conditions = (
				  AnyNodeProcessorsIdleTimeChanged => q(ResourceClass::IBM.Host::EventExpression::"PctTotalTimeIdle!=PctTotalTimeIdle@P"::ImmediateEvaluate::1::Severity::0::ManagementScope::4::NodeNames::"{}"),
				  AnyNodePctRealMemFreeChanged => q(ResourceClass::IBM.Host::EventExpression::"PctRealMemFree!=PctRealMemFree@P"::ImmediateEvaluate::1::Severity::0::ManagementScope::4::NodeNames::"{}"),
				  AnyNodePctTotalPgSpFreeChanged => q(ResourceClass::IBM.Host::EventExpression::"PctTotalPgSpFree!=PctTotalPgSpFree@P"::ImmediateEvaluate::1::Severity::0::ManagementScope::4::NodeNames::"{}"),
				 );
                 #mkrsrc-api IBM.Condition::EventExpression::"PctRealMemFree!=PctRealMemFree@P"::ImmediateEvaluate::"1"::ResourceClass::"IBM.Host"::Name::"AnyNodePctRealMemFreeChanged"::ManagementScope::"4"::Severity::"0"::NodeNames::"{}" 2>&1

# Responses needed
my %responses = ( EchoEventValue => qq(Actions::'{["echoeventvalue",{127},{0},{86400},"/usr/libexec/webmin/csm/perfmon/echoeventvalue",3,0,0,1,{},0]}'), );
# mkresponse -n "echoeventvalue" -o -s "/usr/libexec/webmin/csm/perfmon/echoeventvalue" -e "b" "EchoEventValue" 

# These are the condition and response pairs that should be activated by default.
# This is an hash, where the key consists of '"condition" "response"' 
# The double quotes around condition and response are required, as that is how
# it is displayed by lscondresp
my %condresp = (
				'"AnyNodeProcessorsIdleTimeChanged" "EchoEventValue"' => 1,
				'"AnyNodePctRealMemFreeChanged" "EchoEventValue"' => 1,
				'"AnyNodePctTotalPgSpFreeChanged" "EchoEventValue"' => 1,
			   );

GuiUtils->addHeader('perfmon', 0, 'Peformance Monitoring');

if ($::in{startCheckbox} == 1)     # Establish and start the conditions
  {
	startConditions(\%conditions, \%responses, \%condresp);
  }
elsif (exists($::in{startCheckbox}) && $::in{startCheckbox} == 0)      # stop the conditions
  {
	stopConditions(\%condresp);
  }

my $monitoringStarted = queryConditions(\%conditions, \%responses, \%condresp);

GuiUtils->initEvents($LAST_EVENT_FILE);

# 1st filter out premanaged nodes because a bug in listNodeAttrs doesn't handle them correctly when the
# resource class is set to IBM.Host.
my @nodes = split(/,/, $::in{nodes});
my $where = "Mode='Managed' && Hostname IN ('" . join("','", @nodes) . "')";
$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
#my $newNodeList = NodeUtils->listNodeAttrs(undef, 'Hostname', {WhereStr=>$where});
my $nodeAttrs = NodeUtils->listNodeAttrs(undef, 'Hostname', {WhereStr=>$where});

# Do not actually want to do this, because lsrsrc-api queries IBM.Host on each node 1 at a time right now (Defect 110886)
# Now get the initial performance data for each node
#my $attrs = 'Name,NumProcessors,PctTotalTimeIdle,RealMemSize,PctRealMemFree,TotalPgSpSize,PctTotalPgSpFree';
#$ENV{'CT_MANAGEMENT_SCOPE'} = 3;      # set scope to whole cluster
#my $nodeAttrs = NodeUtils->listNodeAttrs($newNodeList, $attrs, {ResourceClass=>'IBM.Host'});


my $code = GuiUtils->processHTML('perfmon-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('', 'CSM Home');



# Query if the conditions we need for performance monitoring exist and are activated.
# It does this by listing them in the system and then removing them one by one from
# our hashes.  Returns true if they are all created and started.
sub queryConditions
  {
	my ($conditions, $responses, $condresp) = @_;
	
	# Remove any conditions that already exist from our hash
	my @names = keys %$conditions;
	my $select = NodeUtils->buildWhereInList(\@names, 'Name');
	my $outref = GuiUtils->runcmd("lsrsrc-api -i -D ':|:' -s IBM.Condition::${select}::Name");   # Get the current list of conditions
	foreach my $g (@$outref) { delete $$conditions{$g}; }    # Remove all these entries from our hash
	
	# Remove any responses that already exist from our hash
	@names = keys %$responses;
	$select = NodeUtils->buildWhereInList(\@names, 'Name');
	$outref = GuiUtils->runcmd("lsrsrc-api -i -D ':|:' -s IBM.EventResponse::${select}::Name");   # Get the current list of responses
	foreach my $g (@$outref) { delete $$responses{$g}; }    # Remove all these entries from our hash

	# Remove any associations from our hash that are active
	$outref = GuiUtils->runcmd("LANG=C lscondresp -xqd -r EchoEventValue"); # Get the current list of associations and their states
	foreach my $line (@$outref)
	  {
		my ($condition, $response, $node, $state) = split(/:/, $line);
		my $key = "$condition $response";
		#print "<p>$key</p>\n";
		if ($state!~/not/i) { delete $$condresp{$key}; }
		elsif (exists($$condresp{$key})) { $$condresp{$key} = 0; }
	  }
	
	#print "<p>", scalar(%$conditions), "</p>\n";
	#print "<p>", scalar(%$condresp), "</p>\n";
	return !scalar(%$conditions) && !scalar(%$responses) && !scalar(%$condresp); # returns true if we found them all
  }

# Activate the conditions we need for performance monitoring.  Will also create them, if necessary.
sub startConditions
  {
	my ($conditions, $responses, $condresp) = @_;
	#print "<p>here</p>\n";
	
	# queryConditions() will remove the ones that exist from our hashes
	if (queryConditions($conditions, $responses, $condresp)) { return; }       # they are already started
	#print "<p>here2</p>\n";

	# Now create what is left in our hashes
	if (scalar(%$conditions))            # there are some conditions we need to create
	  {
		# Build the mkrsrc-api argument string with all the conditions in it
		my $s = '';
		foreach my $k (keys %$conditions)
		  {
			my $attrs = $$conditions{$k};
			$s .= qq( IBM.Condition::Name::"$k"::$attrs);
		  }
		GuiUtils->runcmd("mkrsrc-api $s");     # create all the conditions
	  }

	if (scalar(%$responses))            # there are some responses we need to create
	  {
		# Build the mkrsrc-api argument string with all the responses in it
		my $s = '';
		foreach my $k (keys %$responses)
		  {
			my $attrs = $$responses{$k};
			$s .= qq( IBM.EventResponse::Name::"$k"::$attrs);
		  }
		GuiUtils->runcmd("mkrsrc-api $s");     # create all the responses
	  }
	
	if (scalar(%$condresp))            # there are some associations we need to create or start
	  {
		foreach my $k (keys %$condresp) { GuiUtils->runcmd("startcondresp $k"); }
	  }
  }

# Stop the conditions we need for performance monitoring.
# Note: this will not work if you call it after queryConditions()
sub stopConditions
  {
	my ($condresp) = @_;
	foreach my $k (keys %$condresp) { GuiUtils->runcmd("stopcondresp $k"); }
  }

sub getParams
  {
	my @nodes;
	my @params;
	foreach my $k (keys %::in)
	  {
		if ($k =~ /^node\d+/) { push @nodes, &urlize($::in{$k}); }
		elsif ($k eq 'historyTimeTextfield') { push @params, "$k=" . &urlize($::in{$k}); }
		#else { push @params, "$k=" . &urlize($::in{$k}); }   # this gets too many
	  }
	#push @params, 'nodes=' . &urlize(join(',', @nodes));  # is it necessary to urlize this?
	if (scalar(@nodes)) { push @params, 'nodes=' . join(',', @nodes); }
	return join('&', @params);
  }


# Get recent events from the audit log.  (All the events since
# the last time this function was called.
sub perfmon
  {
	my ($nodestr) = @_;
	my @nodes = split(/,/, $nodestr);
	my ($sequenceNumber, $eventTime, $time);
	my $resStates = {};
	
	# Get the time of the last event so we know when to check since
	($eventTime, $sequenceNumber) = GuiUtils->retrieveTimeAndSeqNum($LAST_EVENT_FILE);

	# LANG=C /usr/bin/lsaudrec -S ERRM -l -s "SequenceNumber>2684630 && TemplateId>=39 && TemplateId<=51" | grep '^Standard Output *='
	my $out = GuiUtils->runcmd(qq(LANG=C /usr/bin/lsaudrec -x -l -S ERRM -s "SequenceNumber>$sequenceNumber && TemplateId==43 && EventResponseName=='EchoEventValue'" Time SequenceNumber), 0, {NoVerbose=>1});
	#print "out = ", scalar(@$out), "\n";
	foreach my $line (@$out)
	  {
		#print "line=$line\n";
		if ($line =~ /^Time\s*=/) { ($time) = $line=~/^Time\s*=\s*(\S+\s+\S+)/; }    #todo: this time isnt necessarily an event for one of our nodes
		elsif ($line =~ /^Sequence Number\s*=/)
		  {
			($sequenceNumber) = $line=~/^Sequence Number\s*=\s*(\S+)/;
		  }
		elsif ($line =~ /^Standard Output\s*=\s*EchoEventValue/i)
		  {
			my ($output) = $line=~/^Standard Output\s*=\s*(.*)$/;
			my ($respname,$ERRM_COND_NAME,$ERRM_TYPEID,$ERRM_TIME,$ERRM_NODE_NAME,$ERRM_RSRC_CLASS_PNAME,$ERRM_RSRC_NAME,$ERRM_ATTR_PNAME,$ERRM_VALUE) = split(/:\|:/, $output);
			#print "<p>node=$ERRM_NODE_NAME</p>\n";
			if (grep(/^$ERRM_NODE_NAME$/, @nodes))
			  {
				my ($shortname) = split(/\./, $ERRM_NODE_NAME);   # the dots mess up the name of the html object
				$$resStates{"$ERRM_COND_NAME$shortname"} = 100 - $ERRM_VALUE;
			  }
		  }
	  }
	#foreach my $k (keys %$resStates) { print "<p>$k=$$resStates{$k}</p>"; }
	
	# Store the time and sequence # of the last event we got
	if (defined($time)) { $eventTime = GuiUtils->storeTimeAndSeqNum($time, $sequenceNumber, $LAST_EVENT_FILE); }
	#print "</p>\n";

	my $checkedTime = time();
	return ($resStates, $checkedTime, $eventTime);
  }
