#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Display or change properties for a list of nodes

use strict;
use lib $ENV{PWD};
require GuiUtils;

my $MIXED = '(mixed values)';
$ENV{CT_MANAGEMENT_SCOPE}=1;

GuiUtils->init();
#%access=&get_module_acl;

## put in ACL checks here if needed

GuiUtils->addHeader('properties', 0, 'Properties');

my $nodes = $::in{nodes};
delete($::in{nodes});
my %change;
my $changedAttrStr;
if (exists($::in{chgPropButton}))       # they pushed the Change Properties button
  {
	foreach my $key (keys %::in)
	  {
		# Process the original attr values for the Change Properties action
		if ($key =~ /^orig/)
		  {
			#this attribute had an original attribute
			my $attr = $key;
			$attr =~ s/^orig//;
			if (exists $::in{$attr})
			  {
				if ($::in{$key} ne $::in{$attr}) { $change{$attr} = $::in{$attr}; }	# attr was changed
				delete $::in{$attr};
			  }
			delete $::in{$key};
		  }
	  }

	# Get the rest of the attribute values that are non-blank and then
	# run the chnode cmd.
	delete $::in{'chgPropButton'};
	foreach my $key (keys %::in){ #the leftovers are all attributes
	  if(($::in{$key} =~ m/\S/) && ($::in{$key} ne $MIXED) ){ #if it matches a non-whitespace character and was a real value
		$change{$key} = $::in{$key};
	  }
	}
	
	my $chnode_string;
	foreach my $attr (keys %change){
	  $chnode_string .= " $attr=" . NodeUtils->quote($change{$attr});
	}

	if (length($chnode_string))
	  {
		#my $nodes = join ',', @node_array;
		my $outref = GuiUtils->runcmd("$GuiUtils::csmroot/bin/chnode $nodes $chnode_string");
		if ($::config{VERBOSE}) { print "<p>@$outref</p>\n"; }
		if (!$::RUNCMD_RC)
		  {
			$changedAttrStr = 'Properties changed: ' . join(', ', keys %change);
			if (exists($change{Hostname})) { $nodes = $change{Hostname}; }     # have to get attrs below for the new name
		  }
	  }
	else { $changedAttrStr = 'No properties were changed.'; }
  }


# Get the current attribute values
$::VERBOSE = $::GUI_VERBOSE;          # this will cause NodeUtils to show the cmds
# 1st in the ManagedNode class
my @node_array = split(/,/, $nodes);
my $att_ref = NodeUtils->listNodeAttrs(\@node_array, '', {IncludeAttrNames => 1});
#print "<P>scalar=", scalar(@$att_ref), ", att_ref=", $att_ref[0], "</P>\n";

# Now in the Host class on each node
$ENV{'CT_MANAGEMENT_SCOPE'} = 3;      # set scope to whole cluster
my @remoteAttrs = qw(DistributionName DistributionVersion KernelVersion NumProcessors OSName PctRealMemFree PctTotalTimeIdle);

my $att_ref2;
if (scalar(@node_array) == 1)
  {
	# Currently there is a bug in listNodeAttrs ...
	#my $att_ref2 = NodeUtils->listNodeAttrs(\@node_array, $remoteAttrs, {IncludeAttrNames => 1, ResourceClass => 'IBM.Host'});
	my $remoteAttrsStr = join('::', @remoteAttrs);
	my $where = NodeUtils->buildWhereInList(\@node_array, 'NodeNameList');
	# Note: even with -i on lsrsrc-api, when we use NodeNameList in the where string, it will
	#       exit with error if nodes not found.  Prevent the exit with the -1 option to runcmd.
	$att_ref2 = NodeUtils->runcmd("/usr/bin/lsrsrc-api -D ':|:' -n -i -s IBM.Host::${where}::$remoteAttrsStr", -1, 1);
  }
else { $att_ref2 = []; }

$ENV{'CT_MANAGEMENT_SCOPE'} = 1;      # set it back
#print "<p>numlines=", scalar(@$att_ref2), ", $$att_ref2[0]</p>\n";
#push @$att_ref, @$att_ref2;              # add these values to the 1st array


#my %node_attr;
#my %attr_hash;

# Go through the attrs for each node and find common values and which ones are mixed
my %common_attr;
my %orig;
foreach my $node (@$att_ref, @$att_ref2)
  {
	# takes the alternating attrs and values and puts them in the hash correctly
	my %attrs = split(/:\|:/, $node);
	foreach my $a (keys %attrs)
	  {
		my $v = $attrs{$a};
		if (defined($common_attr{$a}))
		  {
			if ($common_attr{$a} eq $v) { next; }
			else
			  {
				$common_attr{$a} = $MIXED;
				if (!grep(/^$a$/, @remoteAttrs)) { $orig{$a} = $MIXED; }       # do not save the readonly attrs from the IBM.Host class
			  }
		  }
		else      # do not have a common attr value for this attr yet
		  {
			$common_attr{$a} = $v;
			if ($v=~/\S/ && !grep(/^$a$/, @remoteAttrs)) { $orig{$a} = $v; }
		  }
	  }
  }

my ($statusImage, $statusAlt);
my $status = $common_attr{Status};
#print "<p>status=$status.</p>\n";
if ($status == 1) { $statusImage = 'green-ball-m.gif'; $statusAlt = 'Alive'; }
elsif ($status == 127 && $common_attr{Mode} eq 'PreManaged') { $statusImage = 'blue-ball-m.gif'; $statusAlt = 'Unconfigured'; }
elsif ($status == 127 || $status eq $MIXED) { $statusImage = 'yellow-ball-m.gif'; $statusAlt = 'Unknown'; }
else { $statusImage = 'red-ball-m.gif'; $statusAlt = 'Unreachable'; }

my ($powerStatusImage, $powerStatusAlt);
my $powerStatus = $common_attr{PowerStatus};
#print "<p>powerStatus=$powerStatus.</p>\n";
if ($powerStatus == 1) { $powerStatusImage = 'green-ball-m.gif'; $powerStatusAlt = 'On'; }
elsif ($powerStatus == 127 || $powerStatus eq $MIXED) { $powerStatusImage = 'yellow-ball-m.gif'; $powerStatusAlt = 'Unknown'; }
elsif ($powerStatus == 128) { $powerStatusImage = 'blue-ball-m.gif'; $powerStatusAlt = 'Unconfigured'; }
else { $powerStatusImage = 'red-ball-m.gif'; $powerStatusAlt = 'Off'; }

my $hostValuesStringClass='Info';
my $cpuLoad = $common_attr{PctTotalTimeIdle};
my ($cpuBlue, $cpuWhite);
if ($cpuLoad ne $MIXED && length($cpuLoad))
  {
	$cpuLoad = sprintf('%3.1f', 100.0 - $cpuLoad);
	$cpuBlue = int($cpuLoad);
	if ($cpuBlue <= 0) { $cpuBlue = 1; }
	if ($cpuBlue >= 100) { $cpuBlue = 99; }
	$cpuWhite = 100 - $cpuBlue;
  }
else
  {
	$cpuBlue = 1;
	$cpuWhite = 99;
	$hostValuesStringClass = 'Error';
  }

my $memUsed = $common_attr{PctRealMemFree};
my ($memBlue, $memWhite);
if ($memUsed ne $MIXED && length($memUsed))
  {
	$memUsed = sprintf('%3.1f', 100.0 - $memUsed);
	$memBlue = int($memUsed);
	if ($memBlue <= 0) { $memBlue = 1; }
	if ($memBlue >= 100) { $memBlue = 99; }
	$memWhite = 100 - $memBlue;
  }
else
  {
	$memBlue = 1;
	$memWhite = 99;
  }

my ($distroImage, $distroAlt);
my $distro = $common_attr{DistributionName};
if (!defined($distro)) { $distroImage = 'invisible.gif';  $distroAlt = ''; }
elsif ($distro=~/red ?hat/i) { $distroImage = 'redhat-s.gif'; $distroAlt = 'RedHat'; }
elsif ($distro=~/suse|sles/i) { $distroImage = 'suse-s.gif'; $distroAlt = 'SuSE/SLES'; }
else { $distroImage = '';  $distroAlt = 'Unknown';}        #todo: make the image a question mark


my $code = GuiUtils->processHTML('properties-src.html');
eval $code;
if (length($@)) { print "<P>$@</P>"; }

&footer('', 'CSM Home');

