#!/usr/bin/perl
#
#    Cluster Systems Management Webmin Module
#    Copyright (C) 2002 by IBM Corp.
#
# Download and update software on the cluster

use strict;
BEGIN { $::CSMDIR = '..'; }
use lib "$::CSMDIR";
require GuiUtils;
GuiUtils->init();

# Go to the page corresponding to the button
#if ($::in{propButton}) { &redirect("cond-props.cgi?" . getParams()); }
my $isAction = keys %::in;

GuiUtils->addHeader(undef, 5, $isAction?'Run Soft Maint Cmds':undef, $isAction);

if ($isAction) { print "<H2>Software Maintenance Commands and Output:</H2>\n"; }

# Set some global variables for versions and directories
my ($distro, $version) = split(/\s/, NodeUtils->distribution());
if (exists($::in{reqrpmsButton}) && $::in{distroVersionTextfield}=~/\S/)
  {
	($distro, $version) = split(/\s/, $::in{distroVersionTextfield});
  }
my $tmpdir = '/tmp/csm';
my $tmpreqsdir = "$tmpdir/reqs";
my $tmpdistrodir = "$tmpdir/$distro/$version";
my $rpmdir = "/csminstall/Linux/$distro/$version/i386/$distro/RPMS";
my $updatedir = "/csminstall/Linux/$distro/$version/i386/updates";
my $autodownload = '/usr/sbin/autoget --noinstall --noupdate --nodldaddinstall --noresolve --nopurge --nocleanup';
my $autoinstall = '/usr/sbin/autoins';


$::GUI_VERBOSE = 1;      # always want to show them the cmds we are running on this page
if (exists($::in{autoupdateButton}))     # download autoupdate
  {
	my $host = 'ftp.mat.univie.ac.at';
	#my $host = '131.130.14.152';
	my $hostdir = '/pub/teschl/autoupdate';
	my $spec = 'autoupdate.spec';
	#my $file = 'autoupdate-4.7.2-1.noarch.rpm';
	GuiUtils->runcmd("/bin/mkdir -p $tmpreqsdir");
	my $errStr;

	# 1st download the spec file so we know the current version of autoupdate
	$::progress_callback_url = $spec;
	ftp_download($host, "$hostdir/$spec", "$tmpreqsdir/$spec", \$errStr, \&progress_callback);
	if (length($errStr)) { print "<P>$errStr</P>\n"; }
	else
	  {
		my $outref = GuiUtils->runcmd(qq(/bin/grep -E "^Version:|^.define vendor_release" $tmpreqsdir/$spec), 0, { NoVerbose => 1 });
		my ($release) = $$outref[0] =~ /^.define vendor_release\s*(\S+)\s*$/;
		my ($version) = $$outref[1] =~ /^Version:\s*(\S+)\s*$/;
		$version .= "-$release";
		print "<p>Latest autoupdate version is $version</p>\n";
		# now download the autoupdate rpm
		my $file = "autoupdate-$version.noarch.rpm";
		$::progress_callback_url = $file;
		#print "<P>Downloading ftp://$host$hostdir/$file ...</P>\n";
		ftp_download($host, "$hostdir/$file", "$tmpreqsdir/$file", \$errStr, \&progress_callback);
		if (length($errStr)) { print "<P>$errStr</P>\n"; }
		else
		  {
			GuiUtils->runcmd('rpm --import /usr/share/rhn/RPM-GPG-KEY');
			GuiUtils->runcmd("/bin/rpm -U $tmpreqsdir/$file");
		  }
	  }
  }
elsif (exists($::in{reqrpmsButton}))     # download RPM prereqs
  {
	GuiUtils->runcmd("/bin/mkdir -p $tmpdistrodir");
	my ($ddir, $vdir, $rpmlist);
	if ($::in{reqrpmsTextfield}=~/\S/) { $rpmlist = $::in{reqrpmsTextfield};  $rpmlist =~ s/,//g; }
	if ($distro eq 'RedHat')
	  {
		$vdir = $version;
		$ddir = $distro;
		if (defined($rpmlist)) {}   # the user specified the rpms to download
		elsif ($version eq '7.2' || $version eq '7.3') { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
		elsif ($version eq '8.0') { $rpmlist = 'glibc glibc-common syslinux freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs'; }
		elsif ($version eq '9') { $rpmlist = 'glibc glibc-common compat-libstdc++ syslinux freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs'; }
		elsif ($version eq 'AS2.1') { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
		else { $rpmlist = 'freetype pdksh perl rdist nfs-utils dhcp expect tcl tk XFree86-libs perl-libnet'; }
	  }
	#elsif ($distro =~ /^RedHatEL/)
	#  {
	#	my ($edition) = $distro =~ /^RedHatEL-(\S+)/;
	#	if ($version eq '3')
	#	  {
	#		$vdir = "enterprise/$version$edition";
	#		$ddir = $distro;
	#		if (!defined($rpmlist)) { $rpmlist = ''; }
	#	  }
	#  }
	else { GuiUtils->msg('E', "Do not support downloads for $distro $version."); }

	if (defined($vdir) && defined($ddir) && defined($rpmlist))
	  {
		my $ftpsite = "http://ftp.redhat.com/pub/redhat/linux/$version/en/os/i386/$distro/RPMS";
		my $updatessite = "http://updates.redhat.com/$version/en/os/i386";
		GuiUtils->runcmd("$autodownload --updatedir $tmpdistrodir --rpmdir $tmpdistrodir --url $updatessite $rpmlist", 1, {NoVerbose=>1});
		GuiUtils->runcmd("$autodownload --updatedir $tmpdistrodir --rpmdir $tmpdistrodir --url $ftpsite $rpmlist", 1, {NoVerbose=>1});
	  }
  }
elsif (exists($::in{installmsButton})) { installms(); }     # install MS
elsif (exists($::in{installectButton})) { installect(); }     # install ECT
elsif (exists($::in{updateCSMWebButton}))        # update this csm web interface
  {
	#todo: get this to work for the external site when it is at the techsupport site
	my $tmpxcsmdir = "$tmpdir/xcsm";
	GuiUtils->runcmd("/bin/mkdir -p $tmpxcsmdir");
	
	my $internalSite = 'http://rs6000.pok.ibm.com/afs/apd.pok.ibm.com/project/csm/install-images/xcsm';
	#my $internalSite = 'http://rs6000.pok.ibm.com/afs/aix.kingston.ibm.com/u/bp/rpms';
	GuiUtils->runcmd("$autoinstall --updatedir $tmpxcsmdir --url $internalSite xcsm.web", 1, {NoVerbose=>1});
  }
elsif (exists($::in{webminButton}))     # download webmin
  {
	my $rpmlist = 'webmin';
	my $ftpsite = 'http://unc.dl.sourceforge.net/sourceforge/webadmin';
	GuiUtils->runcmd("$autodownload --updatedir $updatedir --rpmdir $rpmdir --url $ftpsite $rpmlist", 1, {NoVerbose=>1});
  }
elsif (exists($::in{redhatUpdatesButton}))     # download RedHat updates
  {
	my $ftpsite = "http://ftp.redhat.com/pub/redhat/linux/updates/$version/en/os/i386";
	GuiUtils->runcmd("/usr/sbin/autodld --noinstall --noupdate --nodldaddinstall --updatedir $updatedir --rpmdir $rpmdir --url $ftpsite", 1, {NoVerbose=>1});
  }

if ($isAction)
  {
	print "<P><BR></P>\n";
	&footer('.', 'Software Maint');
  }
else
  {
	my %rpmVersions = GuiUtils->getInstalledVersions(1, qw(csm.server autoupdate csm.ect.web webmin csm.ect));
	my $code = GuiUtils->processHTML('index-src.html');
	eval $code;
	if (length($@)) { print "<P>$@</P>"; }
	&footer('', 'CSM Home');
  }



sub installms
  {
	# Explode tarball
	GuiUtils->msg('I', "Looking in $tmpdir for install images...");
	chdir($tmpdir);
	my $tarballpattern = 'csm-linux-*-GA.*tar.gz';
	my $tarball = GuiUtils->getLatestFile($tarballpattern);
	if (!$tarball) { print "<p>File $tarballpattern not found.  Please download it to /tmp/csm first.</p>\n";  return; }
	my ($version) = $tarball =~ /^csm-linux-(.+?)-/;
	if (!GuiUtils->getLatestFile("csm.server-$version-*.i386.rpm"))
	  {
		# Tarball has not been exploded yet
		GuiUtils->runcmd("/bin/tar -zxvf $tarball", 1, {NoVerbose=>1});
	  }
	else { print "<p>The CSM RPMs have already be extracted from $tarball.</p>\n"; }
	
	# Install csm.core
	my $csmcore = GuiUtils->getLatestFile("csm.core-*.i386.rpm");
	if (!GuiUtils->isInstalled($csmcore))
	  { GuiUtils->runcmd("/bin/rpm -U $csmcore", 1, {NoVerbose=>1}); }
	else { print "<p>$csmcore is already installed.</p>\n"; }
	
	# Run installms
	# The 1 piped into installms is in case another tftp program is installed, we tell it
	# to remove that and install tftp-hpa.
	GuiUtils->runcmd(qq(/bin/echo -e "1\n\n" | $GuiUtils::csmroot/bin/installms -p $tmpdir:$tmpdistrodir), 1);
  }


sub installect
  {
	chdir($tmpdir);
	
	# Install csm.ect.web
	my $ectweb = GuiUtils->getLatestFile("csm.ect.web-*.i386.rpm");
	#todo: have this not complain if it is already installed
	GuiUtils->runcmd("/bin/rpm -U $ectweb", 1, {NoVerbose=>1});
	
	# Install csm.ect
	my $ect = GuiUtils->getLatestFile("csm.ect-*.i386.rpm");
	#todo: have this not complain if it is already installed
	GuiUtils->runcmd("/bin/rpm -U $ect", 1, {NoVerbose=>1});
	
	# Install webmin
	my $webmin = GuiUtils->getLatestFile("$updatedir/webmin-*.noarch.rpm");
	#todo: have this not complain if it is already installed
	GuiUtils->runcmd("/bin/rpm -U $webmin", 1, {NoVerbose=>1});
  }
